package com.deepclone.lw.beans.acm;


import java.net.InetAddress;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.bans.ValidatedBanRequest;
import com.deepclone.lw.cmd.admin.logs.LogLevel;
import com.deepclone.lw.cmd.admin.users.AccountListEntry;
import com.deepclone.lw.cmd.admin.users.AccountSessionEntry;
import com.deepclone.lw.cmd.admin.users.AccountStatus;
import com.deepclone.lw.cmd.admin.users.AccountViewEntry;
import com.deepclone.lw.cmd.admin.users.SessionTerminationType;
import com.deepclone.lw.cmd.admin.users.UserSession;
import com.deepclone.lw.cmd.ext.ListLanguagesResponse;
import com.deepclone.lw.cmd.player.account.BanDetailsResponse;
import com.deepclone.lw.cmd.player.gdata.account.AccountData;
import com.deepclone.lw.cmd.player.gdata.account.PrefCategory;
import com.deepclone.lw.cmd.player.gdata.account.PrefChoice;
import com.deepclone.lw.cmd.player.gdata.account.PrefType;
import com.deepclone.lw.cmd.player.gdata.account.PrefValue;
import com.deepclone.lw.interfaces.acm.*;
import com.deepclone.lw.interfaces.admin.BansDAO;
import com.deepclone.lw.interfaces.eventlog.*;
import com.deepclone.lw.interfaces.i18n.*;
import com.deepclone.lw.interfaces.mailer.*;
import com.deepclone.lw.interfaces.naming.NamingDAO;
import com.deepclone.lw.interfaces.prefs.Preference;
import com.deepclone.lw.interfaces.prefs.PreferenceGroup;
import com.deepclone.lw.interfaces.prefs.PreferenceType;
import com.deepclone.lw.interfaces.prefs.PreferencesDAO;
import com.deepclone.lw.sqld.accounts.*;
import com.deepclone.lw.utils.*;



public class AccountManagementBean
		implements AccountManagement
{

	private TransactionTemplate tTemplate;
	private Mailer mailer;
	private Logger logger;
	private UsersDAO usersDao;
	private UserSessionDAO sessionDao;
	private NamingDAO namingDao;
	private Translator translator;
	private PreferencesDAO prefsDao;
	private BansDAO bansDao;


	@Autowired( required = true )
	public void setTransactionTemplate( PlatformTransactionManager transactionManager )
	{
		this.tTemplate = new TransactionTemplate( transactionManager );
	}


	@Autowired( required = true )
	public void setMailer( Mailer mailer )
	{
		this.mailer = mailer;
	}


	@Autowired( required = true )
	public void setLogger( Logger logger )
	{
		this.logger = logger;
	}


	@Autowired( required = true )
	public void setUsersDao( UsersDAO usersDao )
	{
		this.usersDao = usersDao;
	}


	@Autowired( required = true )
	public void setSessionDao( UserSessionDAO sessionDao )
	{
		this.sessionDao = sessionDao;
	}


	@Autowired( required = true )
	public void setNamingDao( NamingDAO namingDao )
	{
		this.namingDao = namingDao;
	}


	@Autowired( required = true )
	public void setTranslator( Translator translator )
	{
		this.translator = translator;
	}


	@Autowired( required = true )
	public void setPrefsDao( PreferencesDAO prefsDao )
	{
		this.prefsDao = prefsDao;
	}


	@Autowired( required = true )
	public void setBansDao( BansDAO bansDao )
	{
		this.bansDao = bansDao;
	}


	@Override
	public void createAccount( final EmailAddress address , final Password password , final String language )
			throws AccountMailException , MailerException , TranslationException
	{
		SystemLogger sLog = this.logger.getSystemLogger( "AccountManagement" );
		sLog.log( LogLevel.INFO , "Attempting to create account '" + address.getAddress( ) + "'" ).flush( );

		// Create credentials
		AccountOperationResult result;
		result = this.tTemplate.execute( new TransactionCallback< AccountOperationResult >( ) {

			@Override
			public AccountOperationResult doInTransaction( TransactionStatus status )
			{
				AccountOperationResult r = usersDao.createAccount( address , password , language );
				if ( r.getErrorCode( ) != 0 ) {
					status.setRollbackOnly( );
				}
				return r;
			}

		} );

		// Check creation status
		switch ( result.getErrorCode( ) ) {
			case 0:
				break;

			case -1:
				sLog.log( LogLevel.INFO , "Could not create account '" + address.getAddress( ) + "' - it exists" )
						.flush( );
				throw new AccountMailException( );

			case -2:
				sLog
						.log( LogLevel.INFO ,
								"Could not create account '" + address.getAddress( ) + "' - unknown language" ).flush( );
				throw new UnknownLanguageException( language );

			default:
				sLog.log(
						LogLevel.ERROR ,
						"Could not create account '" + address.getAddress( ) + "' - unknown error code "
								+ result.getErrorCode( ) ).flush( );
				throw new RuntimeException( );
		}

		// Send email
		Account account = result.getAccount( );
		MailData data = this.mailer.createMail( language , "registrationMail" , address.getAddress( ) );
		data.setData( "address" , address.getAddress( ) );
		data.setData( "validationKey" , account.getValidationToken( ) );
		try {
			data.sendNow( );
		} catch ( MailerException e ) {
			this.cancelValidation( account );
			sLog.log( LogLevel.INFO , "Could not create account '" + address.getAddress( ) + "'" , e ).flush( );
			if ( e instanceof NotSentException ) {
				throw e;
			}
			throw new RuntimeException( e );
		}
	}


	private void cancelValidation( final Account account )
	{
		this.tTemplate.execute( new TransactionCallbackWithoutResult( ) {
			@Override
			protected void doInTransactionWithoutResult( TransactionStatus status )
			{
				usersDao.cancelAccountValidation( account );
			}
		} );
	}


	@Override
	public void reactivateAccount( final EmailAddress address )
			throws AccountMailException , MailerException
	{
		SystemLogger sLog = this.logger.getSystemLogger( "AccountManagement" );
		sLog.log( LogLevel.INFO , "trying to re-activate account " + address.getAddress( ) ).flush( );

		// Check status
		Account account = this.tTemplate.execute( new TransactionCallback< Account >( ) {

			@Override
			public Account doInTransaction( TransactionStatus status )
			{
				Account r = usersDao.reactivateAccount( address );
				if ( r == null ) {
					status.setRollbackOnly( );
				}
				return r;
			}

		} );
		if ( account == null ) {
			throw new AccountMailException( );
		}

		// Send email
		MailData data;
		try {
			data = this.mailer.createMail( account.getLanguage( ) , "reactivationMail" , address.getAddress( ) );
		} catch ( TranslationException e ) {
			sLog.log( LogLevel.ERROR , "'" + address.getAddress( ) + "' - error while preparing reactivation mail" , e )
					.flush( );
			this.cancelValidation( account );
			throw new RuntimeException( e );
		}

		data.setData( "address" , account.getAddress( ) );
		data.setData( "token" , account.getValidationToken( ) );
		try {
			data.sendNow( );
		} catch ( MailerException e ) {
			this.cancelValidation( account );
			if ( e instanceof NotSentException ) {
				throw e;
			}
			throw new RuntimeException( e );
		}
	}


	@Override
	public ValidationResult validateAccount( final EmailAddress address , final String token , final String empire ,
			final String planet )
	{
		SystemLogger sLog = this.logger.getSystemLogger( "AccountManagement" );
		sLog.log( LogLevel.INFO , "trying to validate account '" + address.getAddress( ) + "'" ).flush( );
		return this.tTemplate.execute( new TransactionCallback< ValidationResult >( ) {

			@Override
			public ValidationResult doInTransaction( TransactionStatus status )
			{
				ValidationResult r = usersDao.validateAccount( address , token , empire , planet );
				if ( r.isError( ) ) {
					status.setRollbackOnly( );
				}
				return r;
			}

		} );
	}


	@Override
	public void requestPasswordRecovery( final EmailAddress address )
			throws AccountMailException , MailerException , PasswordRecoveryException
	{
		SystemLogger sLog = this.logger.getSystemLogger( "AccountManagement" );
		sLog.log( LogLevel.INFO , "'" + address.getAddress( ) + "' requesting password recovery" ).flush( );

		AccountOperationResult result = this.tTemplate.execute( new TransactionCallback< AccountOperationResult >( ) {

			@Override
			public AccountOperationResult doInTransaction( TransactionStatus status )
			{
				AccountOperationResult r = usersDao.requestPasswordRecovery( address );
				if ( r.getErrorCode( ) != 0 ) {
					status.setRollbackOnly( );
				}
				return r;
			}

		} );

		// Throw exceptions as needed
		switch ( result.getErrorCode( ) ) {
			case 0:
				break;

			case 1:
				sLog.log( LogLevel.INFO , "'" + address.getAddress( ) + "' - duplicate password recovery request" )
						.flush( );
				throw new PasswordRecoveryException( false );

			case 2:
				sLog.log( LogLevel.INFO , "'" + address.getAddress( ) + "' - account does not exist" ).flush( );
				throw new AccountMailException( );

			default:
				sLog.log( LogLevel.ERROR , "Unknown error code " + result.getErrorCode( ) ).flush( );
				throw new RuntimeException( );
		}

		// Send email
		MailData data;
		Account account = result.getAccount( );
		try {
			data = this.mailer.createMail( account.getLanguage( ) , "passwordRecoveryMail" , address.getAddress( ) );
		} catch ( TranslationException e ) {
			sLog.log( LogLevel.ERROR , "'" + address.getAddress( ) + "' - error while preparing recovery mail" , e )
					.flush( );
			this.cancelPasswordRecovery( account );
			throw new RuntimeException( e );
		}

		data.setData( "address" , address.getAddress( ) );
		data.setData( "token" , account.getPwdRecoveryToken( ) );

		try {
			data.sendNow( );
		} catch ( MailerException e ) {
			this.cancelPasswordRecovery( account );
			if ( e instanceof NotSentException ) {
				throw e;
			}
			throw new RuntimeException( e );
		}
	}


	private void cancelPasswordRecovery( final Account account )
	{
		this.tTemplate.execute( new TransactionCallbackWithoutResult( ) {
			@Override
			protected void doInTransactionWithoutResult( TransactionStatus status )
			{
				usersDao.cancelPasswordRecovery( account );
			}
		} );
	}


	@Override
	public void recoverPassword( final EmailAddress address , final String token , final Password password )
			throws AccountMailException , PasswordRecoveryException , PasswordProhibitedException
	{
		SystemLogger sLog = this.logger.getSystemLogger( "AccountManagement" );
		sLog.log( LogLevel.INFO , "'" + address.getAddress( ) + "' confirming password recovery" ).flush( );

		AccountOperationResult result = this.tTemplate.execute( new TransactionCallback< AccountOperationResult >( ) {

			@Override
			public AccountOperationResult doInTransaction( TransactionStatus status )
			{
				AccountOperationResult r = usersDao.confirmPasswordRecovery( address , token , password );
				if ( r.getErrorCode( ) != 0 ) {
					status.setRollbackOnly( );
				}
				return r;
			}

		} );

		// Handle errors
		switch ( result.getErrorCode( ) ) {
			case 0:
				break;

			case 1:
				throw new AccountMailException( );

			case 2:
				throw new PasswordProhibitedException( );

			default:
				sLog.log( LogLevel.ERROR , "Unknown error code " + result.getErrorCode( ) ).flush( );
				throw new RuntimeException( );

		}
	}


	@Override
	@Transactional
	public AccountSession login( final EmailAddress address , String challenge , String sha1Hash , String md5Hash ,
			InetAddress ipAddress , String clientType , String sessionName )
	{
		Account account = this.usersDao.getAccount( address );

		// Check account
		if ( account == null ) {
			SystemLogger sLog = this.logger.getSystemLogger( "AccountManagement" );
			sLog.log( LogLevel.INFO , "login attempt for unknown address '" + address.getAddress( ) + "'" ).flush( );
			return null;
		}

		// Check challenge response
		String eSha1 = DigestHelper.digest( "sha-1" , challenge + " " + account.getPassSha1( ) );
		String eMd5 = DigestHelper.digest( "md5" , challenge + " " + account.getPassMd5( ) );

		if ( ! ( eSha1.equals( sha1Hash ) && eMd5.equals( md5Hash ) ) ) {
			return null;
		}

		// Create session
		long sId = this.sessionDao.startSession( account.getId( ) , sessionName , clientType , ipAddress.toString( ) );
		return new AccountSession( account , sId );
	}


	@Override
	@Transactional
	public void logout( long session , SessionTerminationType reason )
	{
		this.sessionDao.endSession( session , reason );
	}


	@Override
	@Transactional
	public Account restoreSession( long session )
	{
		UserSession sData = this.sessionDao.getSession( session );
		if ( sData == null || sData.getEndType( ) != null ) {
			return null;
		}
		return this.usersDao.getAccount( sData.getCredentialsId( ) );
	}


	@Override
	@Transactional
	public Account getAccount( final EmailAddress address )
	{
		return this.usersDao.getAccount( address );
	}


	@Override
	@Transactional
	public List< String > getEmpireNames( final EmailAddress address )
	{
		Account accnt = this.usersDao.getAccount( address );
		if ( accnt == null ) {
			return null;
		}
		return this.namingDao.getEmpireNames( accnt.getId( ) );
	}


	@Override
	@Transactional
	public AccountData getAccountPage( EmailAddress address )
	{
		Account account = this.getAccount( address );
		return this.accountToPage( account );
	}


	private AccountData accountToPage( Account account )
	{
		AccountData data = new AccountData( );
		data.setAddress( account.getAddress( ) );
		data.setGameCredits( account.getGameCredits( ) );
		data.setLanguage( account.getLanguage( ) );
		data.setQuitGame( account.getInactivityStart( ) );
		data.setVacCredits( account.getVacationCredits( ) );
		data.setVacTime( account.getVacationTime( ) );
		data.setVacStart( account.getVacationStart( ) );
		data.setMailChange( this.usersDao.getMailChange( account.getId( ) ) );

		// Get supported languages
		Map< String , String > languages = new HashMap< String , String >( );
		try {
			for ( String lId : this.translator.getSupportedLanguages( ) ) {
				languages.put( lId , this.translator.getLanguageName( lId ) );
			}
		} catch ( UnknownLanguageException e ) {
			throw new RuntimeException( e );
		}
		data.setSupportedLanguages( new ListLanguagesResponse( languages ) );

		// Extract account preferences
		for ( PreferenceGroup group : this.prefsDao.getPreferences( account ).getGroups( ) ) {
			PrefCategory pCat = new PrefCategory( );
			pCat.setName( group.getDisplay( ) );
			for ( Preference pref : group.getPreferences( ) ) {
				PrefValue pValue = new PrefValue( );

				pValue.setId( pref.getName( ) );
				pValue.setName( pref.getDisplayName( ) );
				pValue.setDescription( pref.getDescription( ) );

				PreferenceType< ? > dType = pref.getType( );
				if ( dType.getType( ) == String.class ) {
					pValue.setType( PrefType.STRING );
				} else if ( dType.getType( ) == Integer.class ) {
					pValue.setType( PrefType.INTEGER );
				} else if ( dType.getType( ) == Boolean.class ) {
					pValue.setType( PrefType.BOOLEAN );
				} else {
					Map< String , String > choices = dType.getChoices( );
					List< PrefChoice > cList = new LinkedList< PrefChoice >( );
					for ( Map.Entry< String , String > entry : choices.entrySet( ) ) {
						String name;
						try {
							name = this.translator.translate( data.getLanguage( ) , entry.getValue( ) );
						} catch ( TranslationException e ) {
							throw new RuntimeException( e );
						}
						cList.add( new PrefChoice( entry.getKey( ) , name ) );
					}
					pValue.setType( PrefType.CHOICE );
					pValue.setChoices( cList );
				}

				pValue.setValue( pref.getDBValue( ) );
				pCat.addValue( pValue );
			}
			data.addPreferenceCategory( pCat );
		}

		return data;
	}


	@Override
	@Transactional
	public void setLanguage( EmailAddress address , String language )
	{
		this.usersDao.setLanguage( address , language );
	}


	@Override
	@Transactional
	public boolean setPassword( final EmailAddress address , String challenge , String sha1Hash , String md5Hash ,
			Password newPassword )
			throws PasswordProhibitedException
	{
		Account account = this.usersDao.getAccount( address );

		// Check challenge response
		String eSha1 = DigestHelper.digest( "sha-1" , challenge + " " + account.getPassSha1( ) );
		String eMd5 = DigestHelper.digest( "md5" , challenge + " " + account.getPassMd5( ) );
		if ( ! ( eSha1.equals( sha1Hash ) && eMd5.equals( md5Hash ) ) ) {
			return false;
		}

		if ( !this.usersDao.setPassword( account , newPassword ) ) {
			throw new PasswordProhibitedException( );
		}

		return true;
	}


	@Override
	@Transactional( rollbackFor = {
			RuntimeException.class , AccountMailException.class , MailerException.class
	} )
	public boolean setAddress( EmailAddress address , String challenge , String sha1Hash , String md5Hash ,
			EmailAddress nAddress )
			throws AccountMailException , MailerException
	{
		Account account = this.usersDao.getAccount( address );

		// Check challenge response
		String eSha1 = DigestHelper.digest( "sha-1" , challenge + " " + account.getPassSha1( ) );
		String eMd5 = DigestHelper.digest( "md5" , challenge + " " + account.getPassMd5( ) );
		if ( ! ( eSha1.equals( sha1Hash ) && eMd5.equals( md5Hash ) ) ) {
			return false;
		}

		int errCode = this.usersDao.setAddress( account , nAddress );
		switch ( errCode ) {
			case -1:
				return true;
			case -2:
				throw new AccountMailException( );
		}

		// Send email
		SystemLogger sLog = this.logger.getSystemLogger( "AccountManagement" );
		MailData data;
		account = this.usersDao.getAccount( address );
		try {
			data = this.mailer.createMail( account.getLanguage( ) , "addressChangeMail" , nAddress.getAddress( ) );
		} catch ( TranslationException e ) {
			sLog.log( LogLevel.ERROR , "'" + address.getAddress( ) + "' - error while preparing recovery mail" , e )
					.flush( );
			this.cancelPasswordRecovery( account );
			throw new RuntimeException( e );
		}

		data.setData( "address" , nAddress.getAddress( ) );
		data.setData( "token" , account.getAddressChangeToken( ) );

		try {
			data.sendNow( );
		} catch ( MailerException e ) {
			if ( e instanceof NotSentException ) {
				throw e;
			}
			throw new RuntimeException( e );
		}

		return true;
	}


	@Override
	@Transactional
	public void cancelAddressChange( EmailAddress cAddress )
	{
		this.usersDao.cancelAddressChange( this.usersDao.getAccount( cAddress ) );
	}


	@Override
	@Transactional
	public AccountData confirmAddressChange( EmailAddress cAddress , String code )
	{
		Account account = this.usersDao.getAccount( cAddress );
		if ( account.getAddressChangeToken( ) != null ) {
			int newId = this.usersDao.confirmAddressChange( account , code );
			account = this.usersDao.getAccount( newId );
		}
		return this.accountToPage( account );
	}


	@Override
	@Transactional
	public void resetPreferences( EmailAddress address )
	{
		Account account = this.usersDao.getAccount( address );
		this.prefsDao.resetPreferences( account );
	}


	@Override
	@Transactional
	public void setPreferences( EmailAddress address , Map< String , String > values )
	{
		Account account = this.usersDao.getAccount( address );
		this.prefsDao.setPreferences( account , values );
	}


	@Override
	@Transactional
	public void setQuit( EmailAddress address , String reason )
	{
		Account account = this.usersDao.getAccount( address );
		if ( account.getStatus( ) == AccountStatus.ACTIVE ) {
			this.usersDao.setQuit( account , reason );
		}
	}


	@Override
	@Transactional
	public void cancelQuit( EmailAddress address )
	{
		Account account = this.usersDao.getAccount( address );
		if ( account.getStatus( ) == AccountStatus.QUITTING ) {
			this.usersDao.cancelQuit( account );
		}
	}


	@Override
	@Transactional
	public void toggleVacation( EmailAddress address )
	{
		Account account = this.usersDao.getAccount( address );
		AccountStatus status = account.getStatus( );
		if ( status == AccountStatus.ACTIVE ) {
			this.usersDao.enterVacation( account );
		} else if ( status == AccountStatus.START_VACATION || status == AccountStatus.VACATION ) {
			this.usersDao.leaveVacation( account );
		}
	}


	@Override
	@Transactional
	public List< AccountListEntry > listAccounts( AccountStatus status , boolean online )
	{
		if ( online ) {
			return this.usersDao.listOnlineAccounts( status );
		}
		return this.usersDao.listAccounts( status );
	}


	@Override
	@Transactional
	public AccountViewEntry getAccountView( int id )
	{
		AccountViewEntry basics = this.usersDao.viewAccount( id );
		if ( basics == null ) {
			return null;
		}
		basics.setOnline( this.sessionDao.isOnline( id ) );
		basics.setEmpireNames( this.namingDao.getEmpireNames( id ) );
		return basics;
	}


	@Override
	@Transactional
	public AccountSessionEntry viewSessions( int id )
	{
		AccountViewEntry basics = this.usersDao.viewAccount( id );
		if ( basics == null ) {
			return null;
		}

		AccountSessionEntry entry = new AccountSessionEntry( basics );
		entry.setSessions( this.sessionDao.getSessions( id ) );
		return entry;
	}


	@Override
	@Transactional
	public void giveCredits( Administrator admin , int id , int credits )
	{
		this.usersDao.giveCredits( admin.getId( ) , id , credits );
	}


	@Override
	@Transactional
	public BanDetailsResponse getBanDetails( EmailAddress address )
	{
		ValidatedBanRequest ban = this.bansDao.getActiveBan( address );
		if ( ban == null ) {
			throw new RuntimeException( "account not banned" );
		}
		return new BanDetailsResponse( ban.getUpdate( ) , ban.getReason( ) , ban.isRedeemable( ) );
	}
}
