package com.deepclone.lw.beans.acm;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.admin.users.AccountBanEntry;
import com.deepclone.lw.cmd.admin.users.AccountListEntry;
import com.deepclone.lw.cmd.admin.users.AccountStatus;
import com.deepclone.lw.cmd.admin.users.AccountViewEntry;
import com.deepclone.lw.cmd.player.gdata.account.MailChangeData;
import com.deepclone.lw.interfaces.acm.UsersDAO;
import com.deepclone.lw.sqld.accounts.Account;
import com.deepclone.lw.sqld.accounts.AccountOperationResult;
import com.deepclone.lw.sqld.accounts.QuittingAccount;
import com.deepclone.lw.sqld.accounts.ValidationResult;
import com.deepclone.lw.utils.EmailAddress;
import com.deepclone.lw.utils.Password;
import com.deepclone.lw.utils.StoredProc;



public class UsersDAOBean
		implements UsersDAO
{
	private static abstract class AccountListMapper< T extends AccountListEntry >
			implements RowMapper< T >
	{

		protected void getCommonColumns( AccountListEntry e , ResultSet rs )
				throws SQLException
		{
			e.setId( rs.getInt( "id" ) );
			e.setAddress( rs.getString( "address" ) );
			e.setLanguage( rs.getString( "language" ) );
			e.setStatus( AccountStatus.valueOf( rs.getString( "status" ) ) );
			e.setCurrentEmpire( rs.getString( "current_empire" ) );
		}

	}

	private final RowMapper< Account > accountMapper;
	private final RowMapper< QuittingAccount > quitMapper;
	private final RowMapper< MailChangeData > mailChangeMapper;
	private final AccountListMapper< AccountListEntry > mAccountList;
	private final AccountListMapper< AccountViewEntry > mAccountView;

	private JdbcTemplate dTemplate;

	private StoredProc fCreateAccount;
	private StoredProc fMailFailure;
	private StoredProc fValidate;
	private StoredProc fReactivate;
	private StoredProc fRequestPasswordRecovery;
	private StoredProc fCancelPasswordRecovery;
	private StoredProc fConfirmPasswordRecovery;
	private StoredProc fSetLanguage;
	private StoredProc fSetPassword;
	private StoredProc fSetAddress;
	private StoredProc fCancelAddressChange;
	private StoredProc fConfirmAddressChange;
	private StoredProc fSetQuit;
	private StoredProc fCancelQuit;
	private StoredProc fEnterVacation;
	private StoredProc fLeaveVacation;
	private StoredProc fGrantCredits;


	public UsersDAOBean( )
	{
		this.accountMapper = new RowMapper< Account >( ) {
			@Override
			public Account mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				Account account = new Account( );

				account.setId( rs.getInt( "id" ) );
				account.setAddress( rs.getString( "address" ) );
				account.setLanguage( rs.getString( "language" ) );
				account.setPassMd5( rs.getString( "pass_md5" ) );
				account.setPassSha1( rs.getString( "pass_sha1" ) );
				account.setGameCredits( rs.getInt( "game_credits" ) );
				account.setStatus( AccountStatus.valueOf( rs.getString( "status" ) ) );
				account.setValidationToken( rs.getString( "validation_token" ) );
				account.setPwdRecoveryToken( rs.getString( "pwd_recovery_token" ) );
				account.setAddressChangeToken( rs.getString( "address_change_token" ) );
				account.setNewAddress( rs.getString( "new_address" ) );
				account.setVacationCredits( rs.getInt( "vacation_credits" ) );
				account.setVacationTime( rs.getInt( "vacation_time" ) );
				account.setVacationStart( rs.getTimestamp( "vacation_start" ) );
				account.setInactivityStart( rs.getTimestamp( "inactivity_begin" ) );
				account.setInactivityReason( rs.getString( "inactivity_reason" ) );
				account.setBanRequestId( rs.getInt( "ban_request_id" ) );

				return account;
			}
		};

		this.mailChangeMapper = new RowMapper< MailChangeData >( ) {
			@Override
			public MailChangeData mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				if ( rs.getBoolean( "used" ) ) {
					return new MailChangeData( rs.getTimestamp( "expires" ) );
				}
				return new MailChangeData( rs.getTimestamp( "expires" ) , rs.getString( "new_address" ) );
			}
		};

		this.quitMapper = new RowMapper< QuittingAccount >( ) {
			@Override
			public QuittingAccount mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				QuittingAccount acc = new QuittingAccount( );
				acc.setId( rs.getInt( "id" ) );
				acc.setAddress( rs.getString( "address" ) );
				acc.setLanguage( rs.getString( "language" ) );
				return acc;
			}
		};

		this.mAccountList = new AccountListMapper< AccountListEntry >( ) {
			@Override
			public AccountListEntry mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				AccountListEntry e = new AccountListEntry( );
				this.getCommonColumns( e , rs );
				return e;
			}
		};

		this.mAccountView = new AccountListMapper< AccountViewEntry >( ) {
			@Override
			public AccountViewEntry mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				AccountViewEntry e = new AccountViewEntry( );
				this.getCommonColumns( e , rs );
				e.setGameCredits( rs.getInt( "game_credits" ) );
				e.setVacationCredits( rs.getInt( "vacation_credits" ) );
				if ( !e.getStatus( ).isActive( ) && e.getStatus( ) != AccountStatus.UNCONFIRMED ) {
					e.setStatusStart( rs.getTimestamp( "inactivity_begin" ) );
				} else {
					e.setStatusStart( rs.getTimestamp( "vacation_start" ) );
				}
				e.setInactivityReason( rs.getString( "inactivity_reason" ) );
				e.setEmpireId( (Integer) rs.getObject( "current_empire_id" ) );

				if ( e.getStatus( ) == AccountStatus.BANNED ) {
					AccountBanEntry ban = new AccountBanEntry( );
					ban.setRequestedById( rs.getInt( "ban_req_id" ) );
					ban.setRequestedByName( rs.getString( "ban_req_name" ) );
					ban.setConfirmedById( rs.getInt( "ban_val_id" ) );
					ban.setConfirmedByName( rs.getString( "ban_val_name" ) );
					e.setBan( ban );
				}

				e.setWarnings( rs.getInt( "warnings_count" ) );
				e.setLastWarning( rs.getTimestamp( "warnings_last" ) );

				return e;
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fCreateAccount = new StoredProc( dataSource , "users" , "create_credentials" );
		this.fCreateAccount.addParameter( "address" , Types.VARCHAR );
		this.fCreateAccount.addParameter( "lang_id" , Types.VARCHAR );
		this.fCreateAccount.addParameter( "pass_md5" , Types.VARCHAR );
		this.fCreateAccount.addParameter( "pass_sha1" , Types.VARCHAR );
		this.fCreateAccount.addOutput( "err_code" , Types.INTEGER );
		this.fCreateAccount.addOutput( "a_id" , Types.INTEGER );

		this.fMailFailure = new StoredProc( dataSource , "users" , "validation_mail_failure" );
		this.fMailFailure.addParameter( "a_id" , Types.INTEGER );

		this.fValidate = new StoredProc( dataSource , "users" , "validate" );
		this.fValidate.addParameter( "addr" , Types.VARCHAR );
		this.fValidate.addParameter( "vtoken" , Types.VARCHAR );
		this.fValidate.addParameter( "empname" , Types.VARCHAR );
		this.fValidate.addParameter( "plname" , Types.VARCHAR );
		this.fValidate.addOutput( "account_error" , Types.INTEGER );
		this.fValidate.addOutput( "empire_error" , Types.INTEGER );
		this.fValidate.addOutput( "planet_error" , Types.INTEGER );

		this.fReactivate = new StoredProc( dataSource , "users" , "reactivate" );
		this.fReactivate.addParameter( "addr" , Types.VARCHAR );
		this.fReactivate.addOutput( "success" , Types.BOOLEAN );
		this.fReactivate.addOutput( "a_id" , Types.INTEGER );

		this.fRequestPasswordRecovery = new StoredProc( dataSource , "users" , "request_password_recovery" );
		this.fRequestPasswordRecovery.addParameter( "addr" , Types.VARCHAR );
		this.fRequestPasswordRecovery.addOutput( "err_code" , Types.INTEGER );
		this.fRequestPasswordRecovery.addOutput( "a_id" , Types.INTEGER );

		this.fCancelPasswordRecovery = new StoredProc( dataSource , "users" , "cancel_password_recovery" );
		this.fCancelPasswordRecovery.addParameter( "a_id" , Types.INTEGER );

		this.fConfirmPasswordRecovery = new StoredProc( dataSource , "users" , "confirm_password_recovery" );
		this.fConfirmPasswordRecovery.addParameter( "umail" , Types.VARCHAR );
		this.fConfirmPasswordRecovery.addParameter( "tok" , Types.VARCHAR );
		this.fConfirmPasswordRecovery.addParameter( "pmd5" , Types.VARCHAR );
		this.fConfirmPasswordRecovery.addParameter( "psha1" , Types.VARCHAR );
		this.fConfirmPasswordRecovery.addOutput( "err_code" , Types.INTEGER );
		this.fConfirmPasswordRecovery.addOutput( "a_id" , Types.INTEGER );

		this.fSetLanguage = new StoredProc( dataSource , "users" , "set_language" );
		this.fSetLanguage.addParameter( "umail" , Types.VARCHAR );
		this.fSetLanguage.addParameter( "lang_id" , Types.VARCHAR );

		this.fSetPassword = new StoredProc( dataSource , "users" , "set_password" );
		this.fSetPassword.addParameter( "u_id" , Types.INTEGER );
		this.fSetPassword.addParameter( "pmd5" , Types.VARCHAR );
		this.fSetPassword.addParameter( "psha1" , Types.VARCHAR );
		this.fSetPassword.addOutput( "success" , Types.BOOLEAN );

		this.fSetAddress = new StoredProc( dataSource , "users" , "request_address_change" );
		this.fSetAddress.addParameter( "u_id" , Types.INTEGER );
		this.fSetAddress.addParameter( "n_address" , Types.VARCHAR );
		this.fSetAddress.addOutput( "err_code" , Types.INTEGER );

		this.fCancelAddressChange = new StoredProc( dataSource , "users" , "cancel_address_change" );
		this.fCancelAddressChange.addParameter( "u_id" , Types.INTEGER );

		this.fConfirmAddressChange = new StoredProc( dataSource , "users" , "confirm_address_change" );
		this.fConfirmAddressChange.addParameter( "u_id" , Types.INTEGER );
		this.fConfirmAddressChange.addParameter( "token" , Types.VARCHAR );
		this.fConfirmAddressChange.addOutput( "n_id" , Types.INTEGER );

		this.fSetQuit = new StoredProc( dataSource , "users" , "set_account_quit" );
		this.fSetQuit.addParameter( "accound_id" , Types.INTEGER );
		this.fSetQuit.addParameter( "reason_text" , Types.VARCHAR );

		this.fCancelQuit = new StoredProc( dataSource , "users" , "cancel_account_quit" );
		this.fCancelQuit.addParameter( "account_id" , Types.INTEGER );

		this.fEnterVacation = new StoredProc( dataSource , "users" , "set_vacation" );
		this.fEnterVacation.addParameter( "account_id" , Types.INTEGER );

		this.fLeaveVacation = new StoredProc( dataSource , "users" , "leave_vacation" );
		this.fLeaveVacation.addParameter( "account_id" , Types.INTEGER );

		this.fGrantCredits = new StoredProc( dataSource , "admin" , "grant_user_credits" );
		this.fGrantCredits.addParameter( "admin_id" , Types.INTEGER );
		this.fGrantCredits.addParameter( "account_id" , Types.INTEGER );
		this.fGrantCredits.addParameter( "credits" , Types.INTEGER );
	}


	@Override
	public Account getAccount( EmailAddress address )
	{
		String sql = "SELECT * FROM users.accounts_view WHERE address = ?";
		try {
			return this.dTemplate.queryForObject( sql , this.accountMapper , address.getAddress( ) );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public Account getAccount( int id )
	{
		String sql = "SELECT * FROM users.accounts_view WHERE id = ?";
		try {
			return this.dTemplate.queryForObject( sql , this.accountMapper , id );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	private AccountOperationResult mapToResult( Map< String , Object > m )
	{
		int errCode = (Integer) m.get( "err_code" );

		AccountOperationResult result = new AccountOperationResult( );
		result.setErrorCode( errCode );
		if ( errCode == 0 ) {
			result.setAccount( this.getAccount( (Integer) m.get( "a_id" ) ) );
		}

		return result;
	}


	@Override
	public AccountOperationResult createAccount( EmailAddress address , Password password , String language )
	{
		Map< String , Object > m = this.fCreateAccount.execute( address.getAddress( ) , language , password.getMd5( ) ,
				password.getSha1( ) );
		return this.mapToResult( m );
	}


	@Override
	public void cancelAccountValidation( Account account )
	{
		this.fMailFailure.execute( account.getId( ) );
	}


	@Override
	public ValidationResult validateAccount( EmailAddress address , String token , String eName , String pName )
	{
		Map< String , Object > m = this.fValidate.execute( address.getAddress( ) , token , eName , pName );
		ValidationResult vr = new ValidationResult( );
		vr.setAccountError( (Integer) m.get( "account_error" ) );
		vr.setEmpireError( (Integer) m.get( "empire_error" ) );
		vr.setPlanetError( (Integer) m.get( "planet_error" ) );
		return vr;
	}


	@Override
	public Account reactivateAccount( EmailAddress address )
	{
		Map< String , Object > m = this.fReactivate.execute( address.getAddress( ) );

		boolean success = (Boolean) m.get( "success" );
		if ( !success ) {
			return null;
		}

		return this.getAccount( (Integer) m.get( "a_id" ) );
	}


	@Override
	public AccountOperationResult requestPasswordRecovery( EmailAddress address )
	{
		Map< String , Object > m = this.fRequestPasswordRecovery.execute( address.getAddress( ) );
		return this.mapToResult( m );
	}


	@Override
	public void cancelPasswordRecovery( Account account )
	{
		this.fCancelPasswordRecovery.execute( account.getId( ) );
	}


	@Override
	public AccountOperationResult confirmPasswordRecovery( EmailAddress address , String token , Password nPassword )
	{
		Map< String , Object > m = this.fConfirmPasswordRecovery.execute( address.getAddress( ) , token , nPassword
				.getMd5( ) , nPassword.getSha1( ) );
		return this.mapToResult( m );
	}


	@Override
	public MailChangeData getMailChange( int accountId )
	{
		String sql = "SELECT * FROM users.mail_change_view WHERE id = ?";
		try {
			return this.dTemplate.queryForObject( sql , this.mailChangeMapper , accountId );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public void setLanguage( EmailAddress address , String language )
	{
		this.fSetLanguage.execute( address.getAddress( ) , language );
	}


	@Override
	public boolean setPassword( Account account , Password newPassword )
	{
		int id = account.getId( );
		String sha1 = newPassword.getSha1( );
		String md5 = newPassword.getMd5( );
		return (Boolean) this.fSetPassword.execute( id , sha1 , md5 ).get( "success" );
	}


	@Override
	public int setAddress( Account account , EmailAddress nAddress )
	{
		return (Integer) this.fSetAddress.execute( account.getId( ) , nAddress.getAddress( ) ).get( "err_code" );
	}


	@Override
	public void cancelAddressChange( Account account )
	{
		this.fCancelAddressChange.execute( account.getId( ) );
	}


	@Override
	public int confirmAddressChange( Account account , String code )
	{
		Integer nv;
		nv = (Integer) this.fConfirmAddressChange.execute( account.getId( ) , code ).get( "n_id" );
		return ( nv == null ) ? account.getId( ) : nv.intValue( );
	}


	@Override
	public void expireRequests( )
	{
		this.dTemplate.execute( "SELECT users.expire_requests( )" );
	}


	@Override
	public void setQuit( Account account , String reason )
	{
		this.fSetQuit.execute( account.getId( ) , reason );
	}


	@Override
	public void cancelQuit( Account account )
	{
		this.fCancelQuit.execute( account.getId( ) );
	}


	@Override
	public List< QuittingAccount > processQuits( )
	{
		String sql = "SELECT * FROM users.process_quit_requests( )";
		return this.dTemplate.query( sql , this.quitMapper );
	}


	@Override
	public void enterVacation( Account account )
	{
		this.fEnterVacation.execute( account.getId( ) );
	}


	@Override
	public void leaveVacation( Account account )
	{
		this.fLeaveVacation.execute( account.getId( ) );
	}


	@Override
	public void processVacations( )
	{
		this.dTemplate.execute( "SELECT users.process_vacations( )" );
	}


	@Override
	public List< AccountListEntry > listAccounts( AccountStatus status )
	{
		String sql = "SELECT * FROM admin.users_list";
		Object[] params;
		if ( status != null ) {
			sql += " WHERE status = ?";
			params = new Object[] {
				status.toString( )
			};
		} else {
			params = new Object[] { };
		}
		return this.dTemplate.query( sql , this.mAccountList , params );
	}


	@Override
	public List< AccountListEntry > listOnlineAccounts( AccountStatus status )
	{
		String sql = "SELECT u.* FROM admin.users_list u INNER JOIN users.sessions s ON s.credentials_id = u.id AND s.end_type IS NULL AND s.exclusive";
		Object[] params;
		if ( status != null ) {
			sql += " WHERE u.status = ?";
			params = new Object[] {
				status.toString( )
			};
		} else {
			params = new Object[] { };
		}
		return this.dTemplate.query( sql , this.mAccountList , params );
	}


	@Override
	public AccountViewEntry viewAccount( int id )
	{
		String sql = "SELECT * FROM admin.users_list WHERE id = ?";
		try {
			return this.dTemplate.queryForObject( sql , this.mAccountView , id );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public void giveCredits( int adminId , int accountId , int credits )
	{
		this.fGrantCredits.execute( adminId , accountId , credits );
	}


	@Override
	public void deleteOldAccounts( )
	{
		this.dTemplate.execute( "SELECT users.delete_old_accounts( )" );
	}


	@Override
	public List< QuittingAccount > getInactivesToWarn( )
	{
		String sql = "SELECT * FROM users.check_inactivity_emails( )";
		return this.dTemplate.query( sql , this.quitMapper );
	}


	@Override
	public List< QuittingAccount > getInactivesToDisable( )
	{
		String sql = "SELECT * FROM users.check_inactivity( )";
		return this.dTemplate.query( sql , this.quitMapper );
	}
}
