package com.deepclone.lw.beans.admin;


import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import com.deepclone.lw.cmd.admin.adata.AdminOverview;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.logs.LogEntry;
import com.deepclone.lw.interfaces.admin.AdminDAO;
import com.deepclone.lw.interfaces.eventlog.LogReader;
import com.deepclone.lw.interfaces.mailer.MailData;
import com.deepclone.lw.interfaces.mailer.Mailer;
import com.deepclone.lw.sqld.admin.AdminRecord;



public class AdminRecapTask
		implements Runnable
{

	private final TransactionTemplate tTemplate;
	private final Mailer mailer;
	private final AdminDAO adminDao;
	private final LogReader logReader;

	private Timestamp isTime;
	private List< AdminRecord > admins;
	private final Map< Integer , AdminOverview > overviews = new HashMap< Integer , AdminOverview >( );
	private List< LogEntry > logs;


	public AdminRecapTask( TransactionTemplate tTemplate , Mailer mailer , AdminDAO adminDao , LogReader logReader )
	{
		this.tTemplate = tTemplate;
		this.mailer = mailer;
		this.adminDao = adminDao;
		this.logReader = logReader;
	}


	@Override
	public void run( )
	{
		this.tTemplate.execute( new TransactionCallbackWithoutResult( ) {
			@Override
			protected void doInTransactionWithoutResult( TransactionStatus status )
			{
				checkRecap( );
			}
		} );

		if ( this.isTime == null ) {
			return;
		}

		String logs = this.buildLogsString( );
		for ( AdminRecord admin : this.admins ) {
			if ( !admin.isActive( ) ) {
				continue;
			}

			String mailContents = this.buildMailContents( admin , logs );
			if ( "".equals( mailContents ) ) {
				continue;
			}

			MailData mail;
			try {
				mail = this.mailer.createMail( "en" , "adminRecapMail" , admin.getAddress( ) );
				mail.setData( "contents" , mailContents );
				mail.queue( );
			} catch ( Exception e ) {
				continue;
			}
		}
	}


	private String buildMailContents( AdminRecord admin , String logs )
	{
		AdminOverview overview = this.overviews.get( admin.getId( ) );
		if ( overview == null ) {
			return "";
		}

		StringBuilder mail = new StringBuilder( );
		if ( overview.getNewMessages( ) > 0 ) {
			mail.append( "Unread messages: " ).append( overview.getNewMessages( ) ).append( "\n" );
		}

		int privs = admin.getPrivileges( );
		if ( Privileges.NAME.hasPrivilege( privs ) && overview.getPendingNames( ) > 0 ) {
			mail.append( "Unvalidated map names: " ).append( overview.getPendingNames( ) ).append( "\n" );
		}
		if ( Privileges.BANH.hasPrivilege( privs ) && overview.getPendingBans( ) > 0 ) {
			mail.append( "Pending ban requests: " ).append( overview.getPendingBans( ) ).append( "\n" );
		}
		if ( Privileges.BUGT.hasPrivilege( privs ) ) {
			boolean showBugs = false;
			if ( overview.getPendingBugs( ) > 0 ) {
				mail.append( "Bug reports pending validation: " ).append( overview.getPendingBugs( ) ).append( "\n" );
				showBugs = true;
			}
			if ( overview.getUpdatedBugs( ) > 0 ) {
				mail.append( "Updated bug reports: " ).append( overview.getUpdatedBugs( ) ).append( "\n" );
				showBugs = true;
			}
			if ( showBugs && overview.getOpenBugs( ) > 0 ) {
				mail.append( "Open bug reports: " ).append( overview.getOpenBugs( ) ).append( "\n" );
			}
		}
		if ( !"".equals( logs ) && Privileges.LOGS.hasPrivilege( privs ) ) {
			mail.append( "\n" ).append( logs );
		}

		return mail.toString( );
	}


	private String buildLogsString( )
	{
		if ( this.logs.isEmpty( ) ) {
			return "";
		}

		StringBuilder builder = new StringBuilder( ).append( "Administrative actions log\n\n" );
		SimpleDateFormat dfmt = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss" );
		for ( LogEntry entry : this.logs ) {
			builder.append( dfmt.format( entry.getTimestamp( ) ) );
			builder.append( " - " ).append( entry.getLevel( ).toString( ) );
			builder.append( " - " ).append( entry.getAbout( ) );
			builder.append( " - " ).append( entry.getEntry( ) );
			builder.append( "\n" );
		}

		return builder.toString( );
	}


	private void checkRecap( )
	{
		this.isTime = this.adminDao.isRecapTime( );
		if ( this.isTime == null ) {
			return;
		}

		this.admins = this.adminDao.listAdministrators( );
		for ( AdminOverview ov : this.adminDao.getOverviews( ) ) {
			this.overviews.put( ov.getId( ) , ov );
		}
		this.logs = this.logReader.getAdminLogSince( this.isTime );
	}

}
