package com.deepclone.lw.beans.admin;


import java.net.InetAddress;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.deepclone.lw.interfaces.admin.IpBan;



/**
 * The IP Ban bean specifies a private map container to hold IP Address ban entries. It also
 * provides a public method to check whether a specified IP address is banned and a public method to
 * increaes the counter of a given IP address.
 * 
 * @author tseeker
 * 
 */
public class IpBanBean
		implements IpBan
{

	/**
	 * Static class with the date the ban was last increased and the counter of times increased.
	 */
	private static class BanEntry
	{
		public Date lastIncrease = new Date( );
		public int counter = 1;
	}

	/** Private HashMap containing current IP addresses and their associated BanEntry's */
	private final Map< InetAddress , BanEntry > entries = new HashMap< InetAddress , BanEntry >( );


	/**
	 * Checks if an IP address is banned. An address is banned if an entry already exists for it, if
	 * the ban's duration has been reset in the last hour and the IP address's counter has been
	 * increased 3 or more times. If the ban is more than an hour old it is removed.
	 */
	@Override
	public boolean isBanned( InetAddress address )
	{
		synchronized ( this.entries ) {
			BanEntry entry = this.entries.get( address );
			if ( entry == null ) {
				return false;
			}

			if ( entry.lastIncrease.before( new Date( new Date( ).getTime( ) - 60 * 60 * 1000 ) ) ) {
				this.entries.remove( address );
				return false;
			}

			if ( entry.counter < 3 ) {
				return false;
			}

			entry.lastIncrease = new Date( );
			return true;
		}
	}


	/**
	 * If the given IP address does not already have an entry associated with it, one is created for
	 * it. If it does, the entries counter is increased by 1.
	 */
	@Override
	public void increaseBanCounter( InetAddress address )
	{
		synchronized ( this.entries ) {
			BanEntry entry = this.entries.get( address );
			if ( entry == null ) {
				entry = new BanEntry( );
				this.entries.put( address , entry );
				return;
			}

			entry.counter++;
			entry.lastIncrease = new Date( );
		}
	}

}
