package com.deepclone.lw.beans.bt.es.data;


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;



/**
 * Planet XML record
 * 
 * <p>
 * This class is used in XML dumps to represent the information about a planet. It includes a few
 * basic values, as well as information about buildings, queues and resources.
 * 
 * <p>
 * This record exists since version 1 (B6M1) of XML dumps. However, the following changes were made
 * later.
 * <ul>
 * <li>Version 2 (B6M2):
 * <ul>
 * <li>resource providers added,
 * </ul>
 * </ul>
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
@XStreamAlias( "planet" )
@SuppressWarnings( "serial" )
public class PlanetInformation
		implements Serializable
{

	/** Planet identifier */
	@XStreamAsAttribute
	private Integer id;

	/** Planet name */
	@XStreamAsAttribute
	private String name;

	/** Current population of the planet */
	@XStreamAlias( "population" )
	private Double population;

	/** Current happiness of the planet's population */
	@XStreamAlias( "current-happiness" )
	private Double currentHappiness;

	/** Target happiness of the planet's population */
	@XStreamAlias( "target-happiness" )
	private double targetHappiness;

	/** List of buildings */
	@XStreamAlias( "buildings" )
	private final List< BuildingsInformation > buildings = new ArrayList< BuildingsInformation >( );

	/** Civilian construction queue */
	@XStreamAlias( "civ-queue" )
	private final QueueInformation civilianQueue = new QueueInformation( );

	/** Military construction queue */
	@XStreamAlias( "mil-queue" )
	private final QueueInformation militaryQueue = new QueueInformation( );

	/** Planet resource deltas */
	@XStreamImplicit
	private final ArrayList< ResourceDeltaInformation > resourceDeltas = new ArrayList< ResourceDeltaInformation >( );

	/** List of resource providers */
	@XStreamImplicit( itemFieldName = "resource-provider" )
	private final ArrayList< ResourceProviderInformation > resourceProviders = new ArrayList< ResourceProviderInformation >( );


	/** @return the planet's identifier */
	public Integer getId( )
	{
		return this.id;
	}


	/**
	 * Set the planet's identifier
	 * 
	 * @param id
	 *            the planet's identifier
	 */
	public void setId( int id )
	{
		this.id = id;
	}


	/** @return the planet's name */
	public String getName( )
	{
		return this.name;
	}


	/**
	 * Set the planet's name
	 * 
	 * @param name
	 *            the planet's name
	 */
	public void setName( String name )
	{
		this.name = name;
	}


	/** @return the planet's current population */
	public Double getPopulation( )
	{
		return this.population;
	}


	/**
	 * Set the planet's current population
	 * 
	 * @param population
	 *            the planet's current population
	 */
	public void setPopulation( double population )
	{
		this.population = population;
	}


	/** @return the current happiness of the planet's population */
	public Double getCurrentHappiness( )
	{
		return this.currentHappiness;
	}


	/**
	 * Set the current happiness of the planet's population
	 * 
	 * @param currentHappiness
	 *            the current happiness of the planet's population
	 */
	public void setCurrentHappiness( double currentHappiness )
	{
		this.currentHappiness = currentHappiness;
	}


	/** @return the target happiness of the planet's population */
	public Double getTargetHappiness( )
	{
		return this.targetHappiness;
	}


	/**
	 * Set the target happiness of the planet's population
	 * 
	 * @param targetHappiness
	 *            the target happiness of the planet's population
	 */
	public void setTargetHappiness( double targetHappiness )
	{
		this.targetHappiness = targetHappiness;
	}


	/** @return the list of buildings constructed on the planet */
	public List< BuildingsInformation > getBuildings( )
	{
		return this.buildings;
	}


	/** @return the civilian construction queue */
	public QueueInformation getCivilianQueue( )
	{
		return this.civilianQueue;
	}


	/** @return the military construction queue */
	public QueueInformation getMilitaryQueue( )
	{
		return this.militaryQueue;
	}


	/** @return the list of resource delta records */
	public ArrayList< ResourceDeltaInformation > getResourceDeltas( )
	{
		return this.resourceDeltas;
	}


	/** @return the list of resource provider records */
	public ArrayList< ResourceProviderInformation > getResourceProviders( )
	{
		return this.resourceProviders;
	}

}
