package com.deepclone.lw.beans.eventlog;


import javax.sql.DataSource;

import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.PlatformTransactionManager;

import com.deepclone.lw.interfaces.sys.Ticker;
import com.deepclone.lw.interfaces.sys.Ticker.Frequency;



/**
 * Log clean-up bean.
 * 
 * <p>
 * This bean is responsible for removing old log entries from the various logging tables. It
 * registers a set of constants which determine the maximal age of a log entry depending on its
 * category (administrative, account or system) and starts a ticker task with a high frequency.
 * 
 * @author tseeker
 * 
 */
public class LogCleanerBean
		implements InitializingBean , DisposableBean
{
	/** Ticker bean */
	private Ticker ticker;

	/** Transaction manager */
	private PlatformTransactionManager transactionManager;

	/** Log cleaner task instance */
	private LogCleanerTask cleanerTask;

	private DataSource dataSource;



	/**
	 * Sets the ticker bean reference (DI).
	 * 
	 * @param ticker
	 *            the ticker bean
	 */
	@Autowired( required = true )
	public void setTicker( Ticker ticker )
	{
		this.ticker = ticker;
	}


	/**
	 * Sets the transaction manager reference (DI).
	 * 
	 * @param manager
	 *            the transaction manager
	 */
	@Autowired( required = true )
	public void setTransactionManager( PlatformTransactionManager manager )
	{
		this.transactionManager = manager;
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dataSource = dataSource;
	}


	/**
	 * Registers constants and initialises the log clean-up task.
	 * 
	 * <p>
	 * Once the dependencies have been injected, this method will register the
	 * <em>log.maxAge.admin</em>, <em>log.maxAge.account</em> and <em>log.maxAge.system</em>
	 * constants, setting their default values to 7 days. It will then create the clean-up task
	 * instance, register it as a constants user, and add it to the ticker with a high frequency.
	 */
	@Override
	public void afterPropertiesSet( )
	{
		// Create clean-up task
		this.cleanerTask = new LogCleanerTask( this.dataSource , this.transactionManager );
		this.ticker.registerTask( Frequency.LOW , "Log cleaner" , this.cleanerTask );
	}


	/**
	 * Removes references to the clean-up task.
	 * 
	 * <p>
	 * Before the bean is destroyed, this method will unregister the clean-up task from the
	 * constants manager and remove the local reference.
	 */
	@Override
	public void destroy( )
	{
		this.cleanerTask = null;
	}

}
