package com.deepclone.lw.beans.eventlog;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.logs.ExceptionEntry;
import com.deepclone.lw.cmd.admin.logs.GetEntryResponse;
import com.deepclone.lw.cmd.admin.logs.LogEntry;
import com.deepclone.lw.cmd.admin.logs.LogLevel;
import com.deepclone.lw.cmd.admin.logs.LogType;
import com.deepclone.lw.cmd.admin.logs.TraceEntry;
import com.deepclone.lw.cmd.admin.logs.ViewLogResponse;
import com.deepclone.lw.interfaces.eventlog.ExtendedLogEntry;
import com.deepclone.lw.interfaces.eventlog.LogReader;
import com.deepclone.lw.sqld.sys.ExceptionLog;
import com.deepclone.lw.sqld.sys.StackTraceLog;



public class LogReaderBean
		implements LogReader
{

	private JdbcTemplate dTemplate;
	private TransactionTemplate tTemplate;

	private final RowMapper< LogEntry > mLogEntry;
	private final RowMapper< ExceptionLog > mExceptionLog;
	private final RowMapper< StackTraceLog > mTraceLog;


	public LogReaderBean( )
	{
		this.mLogEntry = new RowMapper< LogEntry >( ) {
			@Override
			public LogEntry mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				LogEntry entry = new LogEntry( );
				entry.setId( (Long) rs.getObject( "id" ) );
				entry.setTimestamp( rs.getTimestamp( "t" ) );
				entry.setAbout( rs.getString( "component" ) );
				entry.setLevel( LogLevel.valueOf( rs.getString( "level" ) ) );
				entry.setEntry( rs.getString( "message" ) );
				entry.setException( (Long) rs.getObject( "exception_id" ) );
				return entry;
			}
		};
		this.mExceptionLog = new RowMapper< ExceptionLog >( ) {
			@Override
			public ExceptionLog mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return new ExceptionLog( rs.getLong( "id" ) , rs.getInt( "depth" ) , rs.getString( "exc_class" ) , rs
						.getString( "message" ) );
			}
		};
		this.mTraceLog = new RowMapper< StackTraceLog >( ) {
			@Override
			public StackTraceLog mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return new StackTraceLog( rs.getLong( "exception_id" ) , rs.getInt( "depth" ) , rs
						.getString( "location" ) , rs.getString( "file_name" ) , (Integer) rs.getObject( "line_number" ) );
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );
	}


	@Autowired( required = true )
	public void setTransactionManager( PlatformTransactionManager tManager )
	{
		this.tTemplate = new TransactionTemplate( tManager );
	}


	@Override
	public ViewLogResponse getLog( final Administrator admin , final LogType type , final long first , final int count ,
			final LogLevel minLevel , final String component , final boolean excOnly )
	{
		if ( excOnly && type != LogType.SYSTEM ) {
			return new ViewLogResponse( admin , 0 , new LinkedList< LogEntry >( ) );
		}

		return this.tTemplate.execute( new TransactionCallback< ViewLogResponse >( ) {
			@Override
			public ViewLogResponse doInTransaction( TransactionStatus status )
			{
				return getLogQueries( admin , type , first , count , minLevel , component , excOnly );
			}
		} );
	}


	private ViewLogResponse getLogQueries( Administrator admin , LogType type , long first , int count ,
			LogLevel minLevel , String component , boolean excOnly )
	{
		String countQuery = this.buildQuery( true , type , first , count , minLevel , component , excOnly );
		Object[] cqParams = this.buildQueryParams( true , first , count , minLevel , component );
		long nEntries = this.dTemplate.queryForLong( countQuery , cqParams );

		String mainQuery = this.buildQuery( false , type , first , count , minLevel , component , excOnly );
		Object[] mqParams = this.buildQueryParams( false , first , count , minLevel , component );
		List< LogEntry > entries = this.dTemplate.query( mainQuery , this.mLogEntry , mqParams );

		return new ViewLogResponse( admin , nEntries , entries );
	}


	private Object[] buildQueryParams( boolean isCount , long first , int count , LogLevel minLevel , String component )
	{
		boolean hasComponent = ( component != null && !"".equals( component ) );
		Object[] params = new Object[ ( hasComponent ? 2 : 1 ) + ( isCount ? 0 : 2 ) ];
		int i = 0;

		params[ i++ ] = minLevel.toString( );
		if ( hasComponent ) {
			params[ i++ ] = "%" + component + "%";
		}
		if ( !isCount ) {
			params[ i++ ] = (Long) first;
			params[ i++ ] = (Integer) count;
		}

		return params;
	}


	private String buildQuery( boolean isCount , LogType type , long first , int count , LogLevel minLevel ,
			String component , boolean excOnly )
	{
		StringBuilder builder = new StringBuilder( );

		// Selection from the right view
		builder.append( "SELECT " );
		if ( isCount ) {
			builder.append( "count(*)" );
		} else {
			builder.append( "*" );
		}
		builder.append( " FROM sys." ).append( type.toString( ).toLowerCase( ) ).append( "_logs_view WHERE " );

		// Minimal log level
		builder.append( "level >= ?::log_level" );

		// Component
		if ( component != null && !"".equals( component ) ) {
			builder.append( " AND lower( component ) LIKE ?" );
		}

		// Exception only
		if ( excOnly ) {
			builder.append( " AND exception_id IS NOT NULL" );
		}

		if ( !isCount ) {
			// Ordering, index, count
			builder.append( " ORDER BY t DESC OFFSET ? LIMIT ?" );
		}

		return builder.toString( );
	}


	@Override
	public GetEntryResponse getEntry( final Administrator admin , final long id )
	{
		return this.tTemplate.execute( new TransactionCallback< GetEntryResponse >( ) {
			@Override
			public GetEntryResponse doInTransaction( TransactionStatus status )
			{
				return doGetEntryResponse( admin , id );
			}
		} );
	}


	private GetEntryResponse doGetEntryResponse( Administrator admin , long id )
	{
		ExtendedLogEntry lEntry = this.doGetEntry( id );
		if ( lEntry == null ) {
			return new GetEntryResponse( admin , false );
		}
		return new GetEntryResponse( admin , lEntry.logEntry , lEntry.exceptions );
	}


	private ExtendedLogEntry doGetEntry( long id )
	{
		String sql = "SELECT * FROM sys.system_logs_view WHERE id = ?";

		// Get log entry
		LogEntry entry;
		try {
			entry = this.dTemplate.queryForObject( sql , this.mLogEntry , id );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}

		// Get exceptions
		List< ExceptionLog > eLog;
		sql = "SELECT * FROM sys.exceptions WHERE log_id = ? ORDER BY depth";
		eLog = this.dTemplate.query( sql , this.mExceptionLog , id );

		Map< Long , ExceptionLog > eLogId = new HashMap< Long , ExceptionLog >( );
		for ( ExceptionLog ee : eLog ) {
			eLogId.put( ee.getId( ) , ee );
		}

		// Get stack trace entries
		sql = "SELECT st.* FROM sys.stack_traces st INNER JOIN sys.exceptions e ON e.id = st.exception_id WHERE e.log_id = ? ORDER BY st.depth DESC";
		for ( StackTraceLog stLog : this.dTemplate.query( sql , this.mTraceLog , id ) ) {
			ExceptionLog ee = eLogId.get( stLog.getExcId( ) );
			ee.addStackTrace( stLog );
		}

		// Generate output
		List< ExceptionEntry > exceptions = new LinkedList< ExceptionEntry >( );
		for ( ExceptionLog ee : eLog ) {
			ExceptionEntry eEntry = new ExceptionEntry( ee.getClassName( ) , ee.getMessage( ) );
			List< TraceEntry > stackTrace = new LinkedList< TraceEntry >( );
			for ( StackTraceLog stLog : ee.getStack( ) ) {
				stackTrace.add( new TraceEntry( stLog.getLocation( ) , stLog.getFileName( ) , stLog.getLine( ) ) );
			}
			eEntry.setTrace( stackTrace );
			exceptions.add( eEntry );
		}

		return new ExtendedLogEntry( entry , exceptions );
	}


	@Override
	public List< ExtendedLogEntry > getErrorEntries( )
	{
		List< ExtendedLogEntry > result = new LinkedList< ExtendedLogEntry >( );
		String sql = "SELECT * FROM admin.get_error_entries( )";
		RowMapper< Long > longLister = new RowMapper< Long >( ) {
			@Override
			public Long mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return rs.getLong( 1 );
			}
		};
		for ( Long id : this.dTemplate.query( sql , longLister ) ) {
			result.add( this.doGetEntry( id ) );
		}
		return result;
	}


	@Override
	public List< LogEntry > getAdminLogSince( Timestamp timestamp )
	{
		String sql = "SELECT * FROM sys.admin_logs_view WHERE t >= ? AND level >= 'INFO'::log_level ORDER BY t DESC";
		return this.dTemplate.query( sql , this.mLogEntry , timestamp );
	}
}
