package com.deepclone.lw.beans.eventlog;


import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;

import javax.sql.DataSource;

import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.support.TransactionTemplate;

import com.deepclone.lw.interfaces.eventlog.LogWriter;
import com.deepclone.lw.sqld.sys.SystemLogEntry;



/**
 * The log writer bean, once initialised, spawns a task while continuously examines a blocking
 * queue. When entries are added to the queue, they are flushed to the database. When the bean
 * container destroys the bean, a queue entry carrying the null value is added to the queue, causing
 * the task to terminate.
 * 
 * @author tseeker
 * 
 */
public class LogWriterBean
		implements LogWriter , InitializingBean , DisposableBean
{

	/** Transaction management instance */
	private TransactionTemplate tTemplate;

	/** JDBC data source */
	private DataSource dataSource;

	/** The queue used for log entries */
	private LinkedBlockingQueue< EntryQueueItem > queue = null;


	/**
	 * Sets the JDBC data source (DI)
	 * 
	 * @param dataSource
	 *            the JDBC data source
	 */
	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dataSource = dataSource;
	}


	/**
	 * Initialises the interface to the transaction manager (DI)
	 * 
	 * @param manager
	 *            the transaction manager to use
	 */
	@Autowired( required = true )
	public void setTransactionManager( PlatformTransactionManager manager )
	{
		this.tTemplate = new TransactionTemplate( manager );
	}


	/* Documentation in LogWriter interface */
	@Override
	synchronized public void addEntries( List< SystemLogEntry > entries )
	{
		if ( this.queue == null ) {
			return;
		}

		LinkedList< EntryQueueItem > items = new LinkedList< EntryQueueItem >( );
		for ( SystemLogEntry e : entries ) {
			if ( e == null ) {
				continue;
			}
			items.add( new EntryQueueItem( e ) );
		}
		this.queue.addAll( items );
	}


	/**
	 * Bean initialisation - creates the queue and spawns the log writer task.
	 */
	@Override
	public void afterPropertiesSet( )
	{
		this.queue = new LinkedBlockingQueue< EntryQueueItem >( );

		LogWriterTask task;
		task = new LogWriterTask( this.queue , this.dataSource , this.tTemplate );

		Thread t;
		t = new Thread( task );
		t.start( );
	}


	/**
	 * Bean destruction - inserts a null entry on the log writer queue, then waits for the task to
	 * terminate.
	 */
	@Override
	public void destroy( )
	{
		this.queue.add( new EntryQueueItem( ) );
		while ( !this.queue.isEmpty( ) ) {
			Thread.yield( );
		}
		synchronized ( this ) {
			this.queue = null;
		}
	}

}
