package com.deepclone.lw.beans.eventlog;


import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;


import com.deepclone.lw.cmd.admin.logs.LogLevel;
import com.deepclone.lw.interfaces.eventlog.SystemLogger;
import com.deepclone.lw.sqld.sys.ExceptionLog;
import com.deepclone.lw.sqld.sys.StackTraceLog;
import com.deepclone.lw.sqld.sys.SystemLogEntry;



/**
 * The system logger implementation generates {@link SystemLogEntry} instances, setting their fields
 * to the proper values and parsing exceptions as required. When flushed it simply uses the logger
 * bean's flush() method to push its contents to the log writer.
 * 
 * @author tseeker
 */
class SystemLoggerImpl
		implements SystemLogger
{

	/** External (log4j) logger */
	private final Logger l4j = Logger.getLogger( SystemLogger.class );

	/** The logger bean */
	private final LoggerBean logger;

	/** The name of the component this system logger is used by */
	private final String component;

	/** The list of entries that have been created but that haven't been flushed to the logger bean */
	private List< SystemLogEntry > entries = new LinkedList< SystemLogEntry >( );


	/** Stores the logger bean reference and the component's name */
	SystemLoggerImpl( LoggerBean logger , String component )
	{
		this.logger = logger;
		this.component = component;
	}


	/* Documentation in SystemLogger interface */
	@Override
	public SystemLogger flush( )
	{
		synchronized ( this.entries ) {
			if ( !this.entries.isEmpty( ) ) {
				this.logger.flush( this.entries );
				this.entries.clear( );
			}
		}
		return this;
	}


	/* Documentation in SystemLogger interface */
	@Override
	public SystemLogger log( LogLevel level , String message )
	{
		this.toLog4j( level , message , null );

		SystemLogEntry entry = this.makeEntry( level , message );
		synchronized ( this.entries ) {
			this.entries.add( entry );
		}

		return this;
	}


	/* Documentation in SystemLogger interface */
	@Override
	public SystemLogger log( LogLevel level , String message , Throwable exception )
	{
		this.toLog4j( level , message , exception );


		SystemLogEntry entry = this.makeEntry( level , message );
		this.parseException( entry , exception );

		synchronized ( this.entries ) {
			this.entries.add( entry );
		}

		return this;
	}


	/**
	 * Logs a message through Log4J (in addition to the DB log).
	 * 
	 * @param level
	 *            log level
	 * @param message
	 *            message to log
	 * @param exception
	 *            optional exception to log along with the message
	 */
	private void toLog4j( LogLevel level , String message , Throwable exception )
	{
		message = this.component + " - " + message;
		switch ( level ) {
			case DEBUG:
				this.l4j.debug( message , exception );
				break;
			case ERROR:
				this.l4j.error( message , exception );
				break;
			case INFO:
				this.l4j.info( message , exception );
				break;
			case TRACE:
				this.l4j.trace( message , exception );
				break;
			case WARNING:
				this.l4j.warn( message , exception );
				break;
		}
	}


	/**
	 * Generates the basic contents of a log entry using a log level and message
	 * 
	 * @param level
	 *            the entry's log level
	 * @param message
	 *            the entry's message
	 * @return a fully usable {@link SystemLogEntry} instance
	 */
	private SystemLogEntry makeEntry( LogLevel level , String message )
	{
		return new SystemLogEntry( level , this.component , message );
	}


	/**
	 * Parses the specified exception so that the ExceptionLog instance and associated data can
	 * later be retrieved.
	 * 
	 * @param exception
	 *            the exception to parse
	 */
	private void parseException( SystemLogEntry logEntry , Throwable exception )
	{
		do {
			ExceptionLog excData;
			excData = new ExceptionLog( logEntry , exception.getClass( ).getCanonicalName( ) , exception.getMessage( ) );
			this.parseTrace( excData , exception );
			exception = exception.getCause( );
		} while ( exception != null );
	}


	/**
	 * This method converts an exception's stack trace into TracebackLog instances, adding them to
	 * the list of associated objects and returning the top-most item.
	 * 
	 * @param logData
	 *            the exception's log entry
	 * @param exception
	 *            the exception whose stack trace is to be converted
	 */
	private void parseTrace( ExceptionLog logData , Throwable exception )
	{
		StackTraceElement[] trace = exception.getStackTrace( );

		for ( int i = trace.length - 1 ; i >= 0 ; i-- ) {
			StackTraceElement e = trace[ i ];
			new StackTraceLog( logData , e.getClassName( ) + "." + e.getMethodName( ) , e.getFileName( ) , e
					.getLineNumber( ) );
		}
	}

}
