package com.deepclone.lw.beans.i18n;


import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.i18n.UnknownLanguageException;
import com.deepclone.lw.interfaces.i18n.UnknownStringException;



/**
 * Translation component
 * 
 * <p>
 * The translator bean's implementation uses the contents of the {@link I18NData} instance, which it
 * only accesses in read-only mode.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TranslatorBean
		implements Translator
{
	private I18NData data;


	/**
	 * Sets the bean's I18N manager (DI)
	 * 
	 * @param manager
	 *            the I18N manager to use
	 */
	@Autowired( required = true )
	public void setI18NManager( I18NManagerBean manager )
	{
		this.data = manager.getData( );
	}


	/* Documentation in Translator interface */
	@Override
	public Set< String > getSupportedLanguages( )
	{
		Set< String > supported = new HashSet< String >( );
		int sCount , lSize;

		this.data.readLock( ).lock( );
		try {
			sCount = this.data.getStringsCount( );
			if ( sCount == 0 ) {
				return supported;
			}

			for ( String lId : this.data.getLanguages( ) ) {
				lSize = this.data.getLanguageSize( lId );
				if ( lSize == sCount ) {
					supported.add( lId );
				}
			}

			return supported;
		} finally {
			this.data.readLock( ).unlock( );
		}
	}


	/* Documentation in Translator interface */
	@Override
	public boolean isLanguageSupported( String language )
	{
		int lSize , sCount;

		this.data.readLock( ).lock( );
		try {
			lSize = this.data.getLanguageSize( language );
			sCount = this.data.getStringsCount( );

			return ( sCount > 0 && lSize == sCount );
		} finally {
			this.data.readLock( ).unlock( );
		}
	}


	/* Documentation in Translator interface */
	@Override
	public String getLanguageName( String language )
			throws UnknownLanguageException
	{
		this.data.readLock( ).lock( );
		try {
			if ( !this.data.isLanguageComplete( language ) ) {
				throw new UnknownLanguageException( language );
			}
			return this.data.getLanguageName( language );
		} finally {
			this.data.readLock( ).unlock( );
		}
	}


	/* Documentation in Translator interface */
	@Override
	public String translate( String language , String string )
			throws UnknownStringException , UnknownLanguageException
	{
		this.data.readLock( ).lock( );
		try {
			if ( !this.data.hasString( string ) ) {
				throw new UnknownStringException( string );
			}
			if ( !this.data.isLanguageComplete( language ) ) {
				throw new UnknownLanguageException( language );
			}
			return this.data.getTranslation( language , string );
		} finally {
			this.data.readLock( ).unlock( );
		}
	}


	/**
	 * Access the store for the specified language then extract translations
	 */
	@Override
	public Map< String , String > translate( String language , Collection< String > strings )
			throws UnknownStringException , UnknownLanguageException
	{
		this.data.readLock( ).lock( );
		try {
			if ( !this.data.isLanguageComplete( language ) ) {
				throw new UnknownLanguageException( language );
			}
			
			LanguageStore store = this.data.getStore( language );
			HashMap< String , String > result = new HashMap< String , String >( );
			for ( String identifier : strings ) {
				String value = store.getTranslation( identifier );
				if ( value == null ) {
					throw new UnknownStringException( identifier );
				}
				result.put( identifier , value );
			}

			return result;
		} finally {
			this.data.readLock( ).unlock( );
		}
	}

}
