package com.deepclone.lw.beans.mailer;


import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.deepclone.lw.interfaces.mailer.AlreadySentException;
import com.deepclone.lw.interfaces.mailer.MailData;
import com.deepclone.lw.interfaces.mailer.MissingDataException;
import com.deepclone.lw.interfaces.mailer.NotSentException;



/**
 * The mail data implementation consists in a single-use message sender that includes templating
 * capabilities.
 * 
 * @author tseeker
 */
class MailDataImpl
		implements MailData
{

	/* Regular expressions used to identify template variables */
	private static final String camelCase = "[a-z]+([A-Z][a-z]*)*";
	private static final String fieldName = camelCase + "(\\." + camelCase + ")*";
	private static final String field = "\\$\\{(" + fieldName + ")\\}";
	private static final Pattern fPattern = Pattern.compile( field );

	/** Queue-handling task */
	private final MailQueueHandler mailQueueTask;

	/** Address of the message's recipient */
	private final String mailTo;

	/** Template string of the message */
	private final String template;

	/** Field values to use when replacing */
	private Map< String , Object > fields = new HashMap< String , Object >( );

	/** Flag indicating whether the message has already been sent */
	private Boolean sent = false;


	/**
	 * Initialise the instance's data then extract variable names from the template string.f
	 * 
	 * @param task
	 *            the queue-handling task
	 * @param target
	 *            the address of the message's recipient
	 * @param template
	 *            the message's template
	 */
	MailDataImpl( MailQueueHandler task , String target , String template )
	{
		this.mailQueueTask = task;
		this.mailTo = target;
		this.template = template;
		this.parseTemplate( );
	}


	/**
	 * Extracts variable names from the template.
	 */
	private void parseTemplate( )
	{
		Matcher m = fPattern.matcher( this.template );
		while ( m.find( ) ) {
			this.fields.put( m.group( 1 ) , null );
		}
	}


	/**
	 * Makes sure a message is never sent twice.
	 * 
	 * @throws AlreadySentException
	 *             if the message had already been sent.
	 */
	private void checkSent( )
			throws AlreadySentException
	{
		synchronized ( this.sent ) {
			if ( this.sent ) {
				throw new AlreadySentException( );
			}
			this.sent = true;
		}
	}


	/**
	 * Prepares the message's data.
	 * 
	 * This method will check that the message has not been sent, then start replacing variables by
	 * their assigned values in the template. It will return the fully-substituted message text.
	 * 
	 * @return the message's text after variable substitution
	 * @throws AlreadySentException
	 *             if the message has already been sent
	 * @throws MissingDataException
	 *             if one of the variables has no value
	 */
	private String prepareMail( )
			throws AlreadySentException , MissingDataException
	{
		this.checkSent( );

		String message = this.template;
		for ( String fName : this.fields.keySet( ) ) {
			Object value = this.fields.get( fName );
			if ( value == null ) {
				synchronized ( this.sent ) {
					this.sent = false;
				}
				throw new MissingDataException( fName );
			}

			String regexp = "\\$\\{";
			regexp += fName.replace( (CharSequence) "." , (CharSequence) "\\." );
			regexp += "\\}";

			Pattern p = Pattern.compile( regexp );
			message = p.matcher( message ).replaceAll( value.toString( ).replace( "$" , "\\$" ) );
		}

		return message;
	}


	/* Documented in MailData interface */
	@Override
	public void queue( )
			throws AlreadySentException , MissingDataException
	{
		this.mailQueueTask.queueMessage( this.mailTo , this.prepareMail( ) );
	}


	/* Documented in MailData interface */
	@Override
	public void sendNow( )
			throws AlreadySentException , NotSentException , MissingDataException
	{
		this.mailQueueTask.sendMessage( this.mailTo , this.prepareMail( ) );
	}


	/* Documented in MailData interface */
	@Override
	public void setData( String identifier , Object value )
	{
		if ( !this.fields.containsKey( identifier ) ) {
			throw new IllegalArgumentException( "unknown field '" + identifier + "'" );
		}
		if ( this.fields.get( identifier ) != null ) {
			throw new IllegalArgumentException( "field '" + identifier + "' already set" );
		}
		if ( value == null ) {
			throw new IllegalArgumentException( "field value may not be null" );
		}
		this.fields.put( identifier , value );
	}

}
