package com.deepclone.lw.beans.mailer;


import java.util.concurrent.LinkedBlockingQueue;

import org.springframework.mail.MailException;
import org.springframework.mail.MailSender;
import org.springframework.mail.SimpleMailMessage;

import com.deepclone.lw.cmd.admin.logs.LogLevel;
import com.deepclone.lw.interfaces.eventlog.SystemLogger;
import com.deepclone.lw.interfaces.mailer.NotSentException;



/**
 * This class implements the queue-handling task of the mailer bean. This task is responsible for
 * sending mail asynchronously, but it also provides a method allowing mails to be sent
 * synchronously by {@link MailData} instances.
 * 
 * @author tseeker
 */
class MailQueueTask
		implements Runnable , MailQueueHandler
{

	/** The bean's system logger */
	private final SystemLogger logger;

	/** The From: address */
	private final String mailFrom;

	/** The mail sender bean */
	private final MailSender mailSender;

	/** The mail queue */
	private LinkedBlockingQueue< MailQueueItem > queue;


	/** Copies the various data and references */
	MailQueueTask( SystemLogger logger , String mailFrom , LinkedBlockingQueue< MailQueueItem > queue ,
			MailSender mailSender )
	{
		this.logger = logger;
		this.mailFrom = mailFrom;
		this.queue = queue;
		this.mailSender = mailSender;
	}


	/**
	 * Stops the queue-handling task.
	 * 
	 * This method inserts a terminator into the queue, then waits for the queue to be
	 * de-initialised (which indicates that the main loop ended).
	 */
	@Override
	public void stop( )
	{
		this.queue.add( new MailQueueTerminator( ) );
		while ( this.queue != null ) {
			Thread.yield( );
		}
	}


	/**
	 * Adds a message to the queue.
	 * 
	 * This method adds a new {@link QueuedMail} instance to the queue, initialising it using the
	 * specified parameters. Should the queueing fail for some reason, the error is logged.
	 * 
	 * @param mailTo
	 *            the address of the email's recipient.
	 * @param message
	 *            the contents of the email
	 */
	@Override
	public void queueMessage( String mailTo , String message )
	{
		try {
			this.queue.put( new QueuedMail( mailTo , message ) );
			this.logger.log( LogLevel.DEBUG , "queued mail to " + mailTo );
		} catch ( InterruptedException e ) {
			// This should never happen, as the queue is non-blocking on this end
			this.logger.log( LogLevel.ERROR , "could not queue mail to " + mailTo , e );
		}
		this.logger.flush( );
	}


	/**
	 * Sends a message.
	 * 
	 * This method splits the message into two parts (subject and body), then initialises the
	 * {@link SimpleMailMessage} instance corresponding to the mail, before finally trying to send
	 * it through the mail sender bean.
	 * 
	 * @param mailTo
	 *            address of the email's recipient
	 * @param message
	 *            contents of the message
	 * @throws NotSentException
	 *             if the mail could not be sent for some reason
	 */
	@Override
	public void sendMessage( String mailTo , String message )
			throws NotSentException
	{
		// Extract title and body from message
		// FIXME: there are better ways to do this
		String title = "";
		int i = 0;
		while ( i < message.length( ) && message.charAt( i ) != '\n' ) {
			title += message.charAt( i );
			i++;
		}
		while ( i < message.length( ) && message.charAt( i ) == '\n' ) {
			i++;
		}
		String body = ( i == message.length( ) ) ? "" : message.substring( i );

		// Write log if the title or the body are empty
		if ( title.equals( "" ) ) {
			this.logger.log( LogLevel.WARNING , "sending email with no title" );
		}
		if ( body.equals( "" ) ) {
			this.logger.log( LogLevel.WARNING , "sending email with empty body" );
		}
		this.logger.flush( );

		// Create actual mail data
		SimpleMailMessage smm = new SimpleMailMessage( );
		smm.setFrom( this.mailFrom );
		smm.setTo( mailTo );
		smm.setSubject( title );
		smm.setText( body );

		// Try to send it
		synchronized ( this.mailSender ) {
			try {
				this.mailSender.send( smm );
			} catch ( MailException e ) {
				this.logger.log( LogLevel.INFO , "could not send mail to " + mailTo , e ).flush( );
				throw new NotSentException( e );
			}
		}
		this.logger.log( LogLevel.DEBUG , "sent mail to " + mailTo ).flush( );
	}


	/**
	 * Main queue-handling loop.
	 * 
	 * This method contains the task's main loop. It constantly polls the queue for incoming
	 * information, sending queued email. All errors while sending messages are ignored. When a
	 * terminator is found on the queue, all remaining messages will be flushed and the main loop
	 * will exit.
	 */
	@Override
	public void run( )
	{
		boolean keepRunning = true;
		LinkedBlockingQueue< MailQueueItem > queue = this.queue;
		while ( keepRunning ) {
			MailQueueItem item;

			// Wait for items to send
			try {
				item = queue.take( );
			} catch ( InterruptedException e ) {
				this.logger.log( LogLevel.WARNING , "mail queue consumer interrupted" ).flush( );
				continue;
			}

			// Handle terminators
			if ( item instanceof MailQueueTerminator ) {
				keepRunning = false;
				this.logger.log( LogLevel.INFO , "termination order received" ).flush( );
			}

			do {
				// Send mail
				if ( item instanceof QueuedMail ) {
					QueuedMail mail = (QueuedMail) item;
					try {
						this.sendMessage( mail.mailTo , mail.message );
					} catch ( NotSentException e ) {
						// Unable to send; it's been logged, so we ignore it
					}
				}
				// If we're terminating, flush the queue
				if ( ! ( keepRunning || queue.isEmpty( ) ) ) {
					boolean fail;
					do {
						try {
							item = queue.take( );
							fail = false;
						} catch ( InterruptedException e ) {
							fail = true;
						}
					} while ( fail );
				}
			} while ( !keepRunning && !queue.isEmpty( ) );
		}

		// Kill the queue
		this.queue = null;
	}

}
