package com.deepclone.lw.beans.game.resources;


import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.deepclone.lw.interfaces.game.resources.MiningSettingsDAO;
import com.deepclone.lw.interfaces.game.resources.ResourcesController;



/**
 * Resources controller component
 * 
 * <p>
 * This component implements all game actions that affect resources or resource-related information,
 * such as mining priorities.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@Transactional
class ResourcesControllerBean
		implements ResourcesController
{

	/** The mining settings access interface */
	private MiningSettingsDAO settings;


	/**
	 * Dependency injector that sets the mining settings access interface
	 * 
	 * @param settings
	 *            the mining settings access interface
	 */
	@Autowired( required = true )
	public void setMiningSettingsDAO( MiningSettingsDAO settings )
	{
		this.settings = settings;
	}


	/**
	 * Update an empire's mining settings
	 * 
	 * <p>
	 * Start a mining settings update for the specified empire, then set each resource-specific
	 * priority, then apply the update. Exit whenever something goes wrong.
	 */
	@Override
	public void updateEmpireSettings( int empireId , Map< String , Integer > settings )
	{
		if ( this.settings.startUpdate( empireId ) ) {
			this.sendMiningSettings( settings );
		}
	}


	/* Documented in interface */
	@Override
	public void togglePlanet( int empire , int planet )
	{
		this.settings.togglePlanet( empire , planet );
	}


	/**
	 * Update a planet's mining settings
	 * 
	 * <p>
	 * Start a mining settings update for the specified planet / empire combination, then set each
	 * resource-specific priority, then apply the update. Exit whenever something goes wrong.
	 */
	@Override
	public void updatePlanetSettings( int empire , int planet , Map< String , Integer > settings )
	{
		if ( this.settings.startUpdate( empire , planet ) ) {
			this.sendMiningSettings( settings );
		}
	}


	/**
	 * Send mining settings to the database
	 * 
	 * <p>
	 * This method is called once a mining settings update has been started (whether it's a
	 * planet-specific or empire-wide update). It sends all settings to the database server, and
	 * aborts if anything goes wrong.
	 * 
	 * @param settings
	 *            the settings to upload
	 */
	private void sendMiningSettings( Map< String , Integer > settings )
	{
		for ( Map.Entry< String , Integer > entry : settings.entrySet( ) ) {
			if ( !this.settings.setNewPriority( entry.getKey( ) , entry.getValue( ) ) ) {
				return;
			}
		}

		this.settings.applyUpdate( );
	}

}
