package com.deepclone.lw.beans.fleets;


import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.cmd.admin.logs.LogLevel;
import com.deepclone.lw.interfaces.eventlog.Logger;
import com.deepclone.lw.interfaces.eventlog.SystemLogger;
import com.deepclone.lw.interfaces.game.BattlesCache;
import com.deepclone.lw.interfaces.sys.Ticker;
import com.deepclone.lw.interfaces.sys.Ticker.Frequency;
import com.deepclone.lw.sqld.game.battle.*;



public class BattlesCacheBean
		implements BattlesCache , Runnable
{
	private static final int cacheTime = 6; // Half an hour

	private static class CacheKey
	{
		private final int empireId;
		private final long battleId;


		public CacheKey( int empireId , long battleId )
		{
			this.empireId = empireId;
			this.battleId = battleId;
		}


		@Override
		public int hashCode( )
		{
			final int prime = 31;
			int result = 1;
			result = prime * result + (int) ( battleId ^ ( battleId >>> 32 ) );
			result = prime * result + empireId;
			return result;
		}


		@Override
		public boolean equals( Object obj )
		{
			if ( this == obj )
				return true;
			if ( obj == null )
				return false;
			if ( getClass( ) != obj.getClass( ) )
				return false;
			CacheKey other = (CacheKey) obj;
			if ( battleId != other.battleId )
				return false;
			if ( empireId != other.empireId )
				return false;
			return true;
		}

	}

	private static class CacheData
	{
		public int timeLeft = BattlesCacheBean.cacheTime;
		public EmpireBattleRecord battle;
		public List< PresenceRecord > presence;
		public Map< Long , Map< Long , ProtagonistRecord >> protagonists;
		public List< EventRecord > events;
		public Map< Boolean , Map< Long , List< EventItemRecord >>> eventItems;
		public Map< Long , List< BuildingHistoryRecord >> buildings;
		public Map< Long , Map< Long , List< ShipHistoryRecord >>> ships;
	};

	private static class ListCache
	{
		public int timeLeft = BattlesCacheBean.cacheTime;
		public Long lastUpdate;
		public List< BattleListRecord > list = new LinkedList< BattleListRecord >( );
	};

	private BattlesDAOBean battlesDao;
	private SystemLogger logger;
	private final Map< CacheKey , CacheData > cache = new HashMap< CacheKey , CacheData >( );
	private final Map< Integer , ListCache > listCache = new HashMap< Integer , ListCache >( );


	@Autowired( required = true )
	public void setBattlesDAO( BattlesDAOBean battlesDao )
	{
		this.battlesDao = battlesDao;
	}


	@Autowired( required = true )
	public void setTicker( Ticker ticker )
	{
		ticker.registerTask( Frequency.LOW , "Battles cache cleaner" , this );
	}


	@Autowired( required = true )
	public void setLogger( Logger logger )
	{
		this.logger = logger.getSystemLogger( "BattlesCache" );
	}


	private void initCache( CacheKey cKey , EmpireBattleRecord record )
	{
		this.logger.log( LogLevel.TRACE ,
				"Adding battle #" + record.getBattleId( ) + " for empire #" + record.getEmpireId( ) + " to the cache" )
				.flush( );

		CacheData data = new CacheData( );
		data.battle = record;
		data.presence = this.battlesDao.getPresence( record );
		data.protagonists = this.battlesDao.getProtagonists( record );
		data.events = this.battlesDao.getEvents( record );
		data.eventItems = this.battlesDao.getEventItems( record );
		data.buildings = this.battlesDao.getBuildingsHistory( record );
		data.ships = this.battlesDao.getShipsHistory( record );
		this.cache.put( cKey , data );
	}


	private CacheData getCached( EmpireBattleRecord record )
	{
		CacheKey cKey = new CacheKey( record.getEmpireId( ) , record.getBattleId( ) );
		CacheData data;
		synchronized ( this.cache ) {
			data = this.cache.get( cKey );
			if ( data != null ) {
				data.timeLeft = BattlesCacheBean.cacheTime;
			}
		}
		return data;
	}


	@Override
	public void run( )
	{

		// Battle data cache
		synchronized ( this.cache ) {
			List< CacheKey > toRemove = new LinkedList< CacheKey >( );
			for ( Map.Entry< CacheKey , CacheData > entry : this.cache.entrySet( ) ) {
				CacheData data = entry.getValue( );
				data.timeLeft--;
				if ( data.timeLeft < 0 ) {
					toRemove.add( entry.getKey( ) );
				}
			}

			for ( CacheKey k : toRemove ) {
				this.cache.remove( k );
			}

			if ( !toRemove.isEmpty( ) ) {
				this.logger.log( LogLevel.DEBUG ,
						"data cache - " + toRemove.size( ) + " entries removed; cache size: " + this.cache.size( ) )
						.flush( );
			}
		}

		// Battle lists cache
		synchronized ( this.listCache ) {
			List< Integer > toRemove = new LinkedList< Integer >( );
			for ( Map.Entry< Integer , ListCache > entry : this.listCache.entrySet( ) ) {
				ListCache data = entry.getValue( );
				data.timeLeft--;
				if ( data.timeLeft < 0 ) {
					toRemove.add( entry.getKey( ) );
				}
			}

			for ( Integer k : toRemove ) {
				this.listCache.remove( k );
			}

			if ( !toRemove.isEmpty( ) ) {
				this.logger.log( LogLevel.DEBUG ,
						"list cache - " + toRemove.size( ) + " entries removed; cache size: " + this.listCache.size( ) )
						.flush( );
			}
		}
	}


	@Override
	public EmpireBattleRecord getBattleRecord( int empireId , long battleId )
	{
		CacheKey cKey = new CacheKey( empireId , battleId );
		synchronized ( this.cache ) {
			CacheData data = this.cache.get( cKey );
			if ( data != null ) {
				data.timeLeft = BattlesCacheBean.cacheTime;
				return data.battle;
			}

			EmpireBattleRecord record = this.battlesDao.getBattleRecord( empireId , battleId );
			if ( record != null && record.getLastTick( ) != null ) {
				this.initCache( cKey , record );
			}
			return record;
		}
	}


	@Override
	public List< PresenceRecord > getPresence( EmpireBattleRecord record )
	{
		CacheData cData = this.getCached( record );
		if ( cData != null ) {
			return cData.presence;
		}
		return this.battlesDao.getPresence( record );
	}


	@Override
	public Map< Long , Map< Long , ProtagonistRecord >> getProtagonists( EmpireBattleRecord record )
	{
		CacheData cData = this.getCached( record );
		if ( cData != null ) {
			return cData.protagonists;
		}
		return this.battlesDao.getProtagonists( record );
	}


	@Override
	public List< EventRecord > getEvents( EmpireBattleRecord record )
	{
		CacheData cData = this.getCached( record );
		if ( cData != null ) {
			return cData.events;
		}
		return this.battlesDao.getEvents( record );
	}


	@Override
	public Map< Boolean , Map< Long , List< EventItemRecord >>> getEventItems( EmpireBattleRecord record )
	{
		CacheData cData = this.getCached( record );
		if ( cData != null ) {
			return cData.eventItems;
		}
		return this.battlesDao.getEventItems( record );
	}


	@Override
	public Map< Long , List< BuildingHistoryRecord >> getBuildingsHistory( EmpireBattleRecord record )
	{
		CacheData cData = this.getCached( record );
		if ( cData != null ) {
			return cData.buildings;
		}
		return this.battlesDao.getBuildingsHistory( record );
	}


	@Override
	public Map< Long , Map< Long , List< ShipHistoryRecord >>> getShipsHistory( EmpireBattleRecord record )
	{
		CacheData cData = this.getCached( record );
		if ( cData != null ) {
			return cData.ships;
		}
		return this.battlesDao.getShipsHistory( record );
	}


	@Override
	public List< BattleListRecord > getBattles( int empireId )
	{
		List< BattleListRecord > results;
		synchronized ( this.listCache ) {
			ListCache lcEntry = this.listCache.get( empireId );
			int nCached;

			if ( lcEntry == null ) {
				// No cached data, get the whole list and add finished battles to the cache
				lcEntry = new ListCache( );
				nCached = 0;
				results = this.battlesDao.getBattles( empireId );

				for ( BattleListRecord record : results ) {
					if ( !record.isFinished( ) ) {
						continue;
					}
					if ( lcEntry.lastUpdate == null ) {
						lcEntry.lastUpdate = record.getLastUpdate( );
					}
					lcEntry.list.add( record );
				}
				if ( !lcEntry.list.isEmpty( ) ) {
					this.listCache.put( empireId , lcEntry );
				}
			} else {
				// Cached data present, get updates
				results = new LinkedList< BattleListRecord >( );
				nCached = lcEntry.list.size( );
				int i = 0;
				for ( BattleListRecord record : this.battlesDao.getBattles( empireId , lcEntry.lastUpdate ) ) {
					if ( !record.isFinished( ) ) {
						results.add( record );
						continue;
					}
					if ( i == 0 ) {
						lcEntry.lastUpdate = record.getLastUpdate( );
					}
					lcEntry.list.add( i++ , record );
				}
				results.addAll( lcEntry.list );
				lcEntry.timeLeft = BattlesCacheBean.cacheTime;
			}

			nCached = lcEntry.list.size( ) - nCached;
			this.logger.log(
					LogLevel.TRACE ,
					"List of battles for empire #" + empireId + ": " + nCached + " new items cached ("
							+ lcEntry.list.size( ) + " total), " + results.size( ) + " items returned" ).flush( );
		}
		return results;
	}
}
