package com.deepclone.lw.beans.msgs;


import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.interfaces.msg.MessageContentCache;
import com.deepclone.lw.interfaces.msg.MessageRecordsDAO;
import com.deepclone.lw.sqld.msgs.EventType;
import com.deepclone.lw.sqld.msgs.EventTypeRecord;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.MessageDataRecord;



public class MessageContentCacheBean
		implements MessageContentCache
{
	private static final int cacheTime = 24;

	private static class CacheKey
	{
		private final boolean internal;
		private final long id;


		public CacheKey( boolean internal , long id )
		{
			this.internal = internal;
			this.id = id;
		}


		@Override
		public int hashCode( )
		{
			final int prime = 31;
			int result = 1;
			result = prime * result + (int) ( id ^ ( id >>> 32 ) );
			result = prime * result + ( internal ? 1231 : 1237 );
			return result;
		}


		@Override
		public boolean equals( Object obj )
		{
			if ( this == obj )
				return true;
			if ( obj == null )
				return false;
			if ( getClass( ) != obj.getClass( ) )
				return false;
			CacheKey other = (CacheKey) obj;
			if ( id != other.id )
				return false;
			if ( internal != other.internal )
				return false;
			return true;
		}

	}

	private static class CacheData
	{
		public int timeLeft = MessageContentCacheBean.cacheTime;
		public MessageDataRecord dataRecord;
	}

	private final Map< CacheKey , CacheData > cache = new HashMap< CacheKey , CacheData >( );
	private MessageRecordsDAO recordsDao;


	@Autowired( required = true )
	public void setRecordsDao( MessageRecordsDAO recordsDao )
	{
		this.recordsDao = recordsDao;
	}


	@Override
	public Map< Long , MessageDataRecord > getContent( List< InboxRecord > messages )
	{
		// Creates cache keys for all records
		Map< Long , CacheKey > mKeys = this.extractKeys( messages );

		// Generate lists of uncached messages
		List< Long > uncachedText = new LinkedList< Long >( );
		List< Long > uncachedEvents = new LinkedList< Long >( );
		synchronized ( this.cache ) {
			this.getCachedRecords( mKeys , uncachedText , uncachedEvents );
		}

		// Fetch missing data
		List< MessageDataRecord > nRecords = new LinkedList< MessageDataRecord >( );
		if ( !uncachedText.isEmpty( ) ) {
			nRecords.addAll( this.recordsDao.getTextMessages( uncachedText ) );
		}
		if ( !uncachedEvents.isEmpty( ) ) {
			this.getMissingEvents( uncachedEvents , nRecords );
		}

		Map< Long , MessageDataRecord > result = new HashMap< Long , MessageDataRecord >( );
		synchronized ( this.cache ) {
			// Add new records to the cache
			for ( MessageDataRecord record : nRecords ) {
				CacheKey key = new CacheKey( record.isInternal( ) , record.getId( ) );
				CacheData data = new CacheData( );
				data.dataRecord = record;
				this.cache.put( key , data );
			}

			// Extract all required data
			for ( Map.Entry< Long , CacheKey > entry : mKeys.entrySet( ) ) {
				CacheKey key = entry.getValue( );
				result.put( entry.getKey( ) , this.cache.get( key ).dataRecord );
			}
		}

		return result;
	}


	private Map< Long , CacheKey > extractKeys( List< InboxRecord > messages )
	{
		Map< Long , CacheKey > mKeys = new HashMap< Long , CacheKey >( );
		for ( InboxRecord record : messages ) {
			CacheKey key = new CacheKey( record.isInternal( ) , record.getContentId( ) );
			mKeys.put( record.getId( ) , key );
		}
		return mKeys;
	}


	private void getCachedRecords( Map< Long , CacheKey > mKeys , List< Long > uncachedText ,
			List< Long > uncachedEvents )
	{
		for ( Map.Entry< Long , CacheKey > entry : mKeys.entrySet( ) ) {
			CacheKey key = entry.getValue( );
			CacheData data = this.cache.get( key );
			if ( data == null ) {
				( key.internal ? uncachedEvents : uncachedText ).add( key.id );
			} else {
				data.timeLeft = MessageContentCacheBean.cacheTime;
			}
		}
	}


	private void getMissingEvents( List< Long > uncachedEvents , List< MessageDataRecord > nRecords )
	{
		// Determine event types
		Map< EventType , List< Long >> eventsByType = new HashMap< EventType , List< Long > >( );
		for ( EventTypeRecord eType : this.recordsDao.getEventTypes( uncachedEvents ) ) {
			List< Long > typeIds = eventsByType.get( eType.getType( ) );
			if ( typeIds == null ) {
				typeIds = new LinkedList< Long >( );
				eventsByType.put( eType.getType( ) , typeIds );
			}
			typeIds.add( eType.getId( ) );
		}

		// Fetch data
		for ( Map.Entry< EventType , List< Long > > entry : eventsByType.entrySet( ) ) {
			nRecords.addAll( this.recordsDao.getEvents( entry.getKey( ) , entry.getValue( ) ) );
		}
	}


	@Override
	public void clearCache( )
	{
		synchronized ( this.cache ) {
			List< CacheKey > toRemove = new LinkedList< CacheKey >( );
			for ( Map.Entry< CacheKey , CacheData > entry : this.cache.entrySet( ) ) {
				CacheData data = entry.getValue( );
				data.timeLeft--;
				if ( data.timeLeft < 0 ) {
					toRemove.add( entry.getKey( ) );
				}
			}

			for ( CacheKey key : toRemove ) {
				this.cache.remove( key );
			}
		}

	}

}
