package com.deepclone.lw.beans.sys;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.sql.DataSource;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.core.simple.SimpleJdbcCall;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import com.deepclone.lw.cmd.admin.logs.LogLevel;
import com.deepclone.lw.interfaces.eventlog.Logger;
import com.deepclone.lw.interfaces.eventlog.SystemLogger;
import com.deepclone.lw.interfaces.sys.ConstantDefinition;
import com.deepclone.lw.interfaces.sys.ConstantsUser;
import com.deepclone.lw.interfaces.sys.InvalidConstantValue;
import com.deepclone.lw.interfaces.sys.UnknownConstantError;
import com.deepclone.lw.sqld.sys.Constant;
import com.deepclone.lw.utils.StoredProc;



/**
 * This class defines the shared data structures used by both the constants manager bean and its
 * administrative session instances. It also encapsulates all of the code used by both.
 * 
 * @author tseeker
 */
class ConstantsData
{
	/** Database interface */
	private JdbcTemplate dTemplate;

	/** Transaction manager interface */
	private TransactionTemplate tTemplate;

	/** System logger instance for the ConstantsManager component */
	private SystemLogger sysLog;

	/** All registered constants, by name */
	private Map< String , Constant > registeredConstants;

	/** Constants users that have received their initial notification */
	private Set< ConstantsUser > notifiedUsers = new HashSet< ConstantsUser >( );

	/** Constants needed by each user */
	private Map< ConstantsUser , Set< String >> userConstants = new HashMap< ConstantsUser , Set< String > >( );

	/** Users listening to each constant */
	private Map< String , Set< ConstantsUser >> constantUsers = new HashMap< String , Set< ConstantsUser > >( );

	/** Constants by category */
	private Map< String , List< Constant >> categories;

	private SimpleJdbcCall uocConstantNoBounds;
	private SimpleJdbcCall uocConstantSingleBound;
	private SimpleJdbcCall uocConstantTwoBounds;
	private StoredProc fSetConstant;


	/**
	 * Initialises the various references, creates the system logger used by the instance, then
	 * loads all existing constants.
	 * 
	 * @param dataSource
	 *            database interface
	 * @param tTemplate
	 *            transaction manager interface
	 * @param logger
	 *            logger bean interface
	 */
	ConstantsData( DataSource dataSource , TransactionTemplate tTemplate , Logger logger )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.uocConstantNoBounds = new SimpleJdbcCall( dataSource );
		this.uocConstantNoBounds.withCatalogName( "sys" ).withFunctionName( "uoc_constant" );
		this.uocConstantNoBounds.withoutProcedureColumnMetaDataAccess( );
		this.uocConstantNoBounds.addDeclaredParameter( new SqlParameter( "cname" , Types.VARCHAR ) );
		this.uocConstantNoBounds.addDeclaredParameter( new SqlParameter( "cdesc" , Types.VARCHAR ) );
		this.uocConstantNoBounds.addDeclaredParameter( new SqlParameter( "catname" , Types.VARCHAR ) );
		this.uocConstantNoBounds.addDeclaredParameter( new SqlParameter( "value" , Types.REAL ) );

		this.uocConstantSingleBound = new SimpleJdbcCall( dataSource );
		this.uocConstantSingleBound.withCatalogName( "sys" ).withFunctionName( "uoc_constant" );
		this.uocConstantSingleBound.withoutProcedureColumnMetaDataAccess( );
		this.uocConstantSingleBound.addDeclaredParameter( new SqlParameter( "cname" , Types.VARCHAR ) );
		this.uocConstantSingleBound.addDeclaredParameter( new SqlParameter( "cdesc" , Types.VARCHAR ) );
		this.uocConstantSingleBound.addDeclaredParameter( new SqlParameter( "catname" , Types.VARCHAR ) );
		this.uocConstantSingleBound.addDeclaredParameter( new SqlParameter( "value" , Types.REAL ) );
		this.uocConstantSingleBound.addDeclaredParameter( new SqlParameter( "boundary" , Types.REAL ) );
		this.uocConstantSingleBound.addDeclaredParameter( new SqlParameter( "is_min" , Types.BOOLEAN ) );

		this.uocConstantTwoBounds = new SimpleJdbcCall( dataSource );
		this.uocConstantTwoBounds.withCatalogName( "sys" ).withFunctionName( "uoc_constant" );
		this.uocConstantTwoBounds.withoutProcedureColumnMetaDataAccess( );
		this.uocConstantTwoBounds.addDeclaredParameter( new SqlParameter( "cname" , Types.VARCHAR ) );
		this.uocConstantTwoBounds.addDeclaredParameter( new SqlParameter( "cdesc" , Types.VARCHAR ) );
		this.uocConstantTwoBounds.addDeclaredParameter( new SqlParameter( "catname" , Types.VARCHAR ) );
		this.uocConstantTwoBounds.addDeclaredParameter( new SqlParameter( "value" , Types.REAL ) );
		this.uocConstantTwoBounds.addDeclaredParameter( new SqlParameter( "min" , Types.REAL ) );
		this.uocConstantTwoBounds.addDeclaredParameter( new SqlParameter( "max" , Types.REAL ) );

		this.fSetConstant = new StoredProc( dataSource , "sys" , "set_constant" );
		this.fSetConstant.addParameter( "cname" , Types.VARCHAR );
		this.fSetConstant.addParameter( "value" , Types.REAL );
		this.fSetConstant.addParameter( "admin" , Types.INTEGER );

		this.tTemplate = tTemplate;
		this.sysLog = logger.getSystemLogger( "ConstantsManager" );

		this.tTemplate.execute( new TransactionCallbackWithoutResult( ) {

			@Override
			protected void doInTransactionWithoutResult( TransactionStatus status )
			{
				loadConstants( );
			}

		} );
	}


	private void loadConstants( )
	{
		String sql = "SELECT category , name , description , value , min , max FROM sys.constants_view";
		RowMapper< Constant > mapper = new RowMapper< Constant >( ) {

			@Override
			public Constant mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				Constant c = new Constant( );
				c.setCategory( rs.getString( "category" ) );
				c.setName( rs.getString( "name" ) );
				c.setDescription( rs.getString( "description" ) );
				c.setValue( rs.getDouble( "value" ) );
				c.setMinValue( (Float) rs.getObject( "min" ) );
				c.setMaxValue( (Float) rs.getObject( "max" ) );
				return c;
			}

		};

		this.registeredConstants = new HashMap< String , Constant >( );
		this.categories = new HashMap< String , List< Constant > >( );
		for ( Constant cnst : this.dTemplate.query( sql , mapper ) ) {
			this.registeredConstants.put( cnst.getName( ) , cnst );

			List< Constant > category = this.categories.get( cnst.getCategory( ) );
			if ( category == null ) {
				category = new LinkedList< Constant >( );
				this.categories.put( cnst.getCategory( ) , category );
			}
			category.add( cnst );
		}
	}


	private void updateConstants( List< ConstantDefinition > toUpdate )
	{
		for ( ConstantDefinition cDef : toUpdate ) {
			if ( cDef.maxValue == null && cDef.minValue == null ) {
				this.uocConstantNoBounds.execute( cDef.name , cDef.description , cDef.category , cDef.defaultValue );
			} else if ( cDef.maxValue == null ) {
				this.uocConstantSingleBound.execute( cDef.name , cDef.description , cDef.category , cDef.defaultValue ,
						cDef.minValue , true );
			} else if ( cDef.minValue == null ) {
				this.uocConstantSingleBound.execute( cDef.name , cDef.description , cDef.category , cDef.defaultValue ,
						cDef.maxValue , false );
			} else {
				this.uocConstantTwoBounds.execute( cDef.name , cDef.description , cDef.category , cDef.defaultValue ,
						cDef.minValue , cDef.maxValue );
			}
		}
	}


	/**
	 * Checks that the descriptive parts of a constant data instance match the corresponding fields
	 * in a constant definition instance.
	 * 
	 * @param constant
	 *            the constant data instance
	 * @param def
	 *            the definition
	 * @return <code>true</code> if the descriptive parts are a match
	 */
	private boolean compareConstantDescription( Constant constant , ConstantDefinition def )
	{
		return constant.getCategory( ).equals( def.category ) && constant.getDescription( ).equals( def.description );
	}


	/**
	 * Checks that the bounds defined in a constant data instance are identical to the bounds from a
	 * constant definition.
	 * 
	 * @param constant
	 *            the constant data instance
	 * @param def
	 *            the definition
	 * @return <code>true</code> if the bounds match
	 */
	private boolean compareConstantBounds( Constant constant , ConstantDefinition def )
	{
		return constant.getMinValue( ) == def.minValue && constant.getMaxValue( ) == def.maxValue;
	}


	/**
	 * Updates a constant's value after bounds changed.
	 * 
	 * This method, called when a constant data instance's bounds have been changed, verifies that
	 * the constant's current value matches the new bounds and modifies it if that is required.
	 * 
	 * @param constant
	 *            the constant data instance
	 * @param def
	 *            the definition
	 * @return <code>true</code> if the value was updated
	 */
	private boolean updateValue( Constant constant , ConstantDefinition def )
	{
		if ( def.minValue != null && def.minValue > constant.getValue( ) ) {
			constant.setValue( def.minValue );
			return true;
		} else if ( def.maxValue != null && def.maxValue < constant.getValue( ) ) {
			constant.setValue( def.maxValue );
			return true;
		}
		return false;
	}


	/**
	 * Notifies constant users that a specific constant value has been modified.
	 * 
	 * @param changed
	 *            the constant's name
	 * 
	 * @see #notify(Set)
	 */
	private void notify( String changed )
	{
		Set< String > temp = new HashSet< String >( );
		temp.add( changed );
		this.notify( temp );
	}


	/**
	 * Notifies constant users that a set of constants has changed.
	 * 
	 * Users of each specified constant are identified, and separated in two groups: users that have
	 * not received their initial notification and for which initial notification must be attempted,
	 * and already initialised users which will only receive updates on constants that actually
	 * changed.
	 * 
	 * @param changed
	 *            the names of the constants that have been changed.
	 * 
	 * @see #notifyUpdate(ConstantsUser, Set)
	 * @see #attemptFirstNotification(ConstantsUser)
	 */
	private void notify( Set< String > changed )
	{
		Map< ConstantsUser , Set< String >> notifyChange = new HashMap< ConstantsUser , Set< String > >( );
		Set< ConstantsUser > firstNotification = new HashSet< ConstantsUser >( );

		// Build a map of users <-> notifications and a set of first notifications to attempt
		for ( String ccn : changed ) {
			Set< ConstantsUser > users = this.constantUsers.get( ccn );
			if ( users == null ) {
				continue;
			}

			for ( ConstantsUser user : users ) {
				if ( this.notifiedUsers.contains( user ) ) {
					Set< String > userChange = notifyChange.get( user );
					if ( userChange == null ) {
						userChange = new HashSet< String >( );
						notifyChange.put( user , userChange );
					}
					userChange.add( ccn );
				} else {
					firstNotification.add( user );
				}
			}
		}

		// Attempt first notifications
		for ( ConstantsUser user : firstNotification ) {
			this.attemptFirstNotification( user );
		}

		// Send updates to the others
		for ( ConstantsUser user : notifyChange.keySet( ) ) {
			this.notifyUpdate( user , notifyChange.get( user ) );
		}
	}


	/**
	 * Notifies a constant user of changes.
	 * 
	 * This method retrieves the values of the listed constants, then notifies the specified user
	 * that they have changed. If an exception occurs in the user's notification method, an error is
	 * written to the server's log.
	 * 
	 * @param user
	 *            the constants user instance.
	 * @param changed
	 *            the names of the constants for which a notification is to be sent.
	 */
	private void notifyUpdate( ConstantsUser user , Set< String > changed )
	{
		Map< String , Double > values = new HashMap< String , Double >( );

		// Find updated values
		for ( String cn : changed ) {
			values.put( cn , this.registeredConstants.get( cn ).getValue( ) );
		}

		// Send notification
		try {
			user.setConstants( false , values );
		} catch ( Throwable t ) {
			this.sysLog.log( LogLevel.ERROR , "updating constant values caused an exception" , t ).flush( );
		}
	}


	/**
	 * Attempts initial notification of constants values to a constants user.
	 * 
	 * This method is called when constants registered to a not-yet-initialised constants user
	 * change. It attempts to retrieve the required values, returning directly if a constant is
	 * missing. If all values exist, the initial notification is sent. Should an exception occur, it
	 * is logged as an error and the method ends; otherwise the user is added to the set of
	 * initialised users.
	 * 
	 * @param user
	 *            the constants user to notify
	 */
	private void attemptFirstNotification( ConstantsUser user )
	{
		Map< String , Double > values = new HashMap< String , Double >( );

		// Try to find all required values
		for ( String cn : this.userConstants.get( user ) ) {
			Constant cst = this.registeredConstants.get( cn );
			if ( cst == null ) {
				return;
			}
			values.put( cn , cst.getValue( ) );
		}

		// Send initial notification
		try {
			user.setConstants( true , values );
		} catch ( Throwable t ) {
			this.sysLog.log( LogLevel.ERROR , "registering initial constant values caused an exception" , t ).flush( );
			return;
		}
		this.notifiedUsers.add( user );
	}


	/**
	 * Updates the list of categories from the list of constants.
	 * 
	 * This method updates the list of categories by going through all known constants, creating an
	 * entry in the categories map for each category and associating it with the list of constants
	 * it contains.
	 */
	private void updateCategories( )
	{
		if ( this.categories != null ) {
			return;
		}
		this.categories = new HashMap< String , List< Constant > >( );
		for ( Constant c : this.registeredConstants.values( ) ) {
			String cat = c.getCategory( );
			List< Constant > cContents;
			cContents = this.categories.get( cat );
			if ( cContents == null ) {
				cContents = new LinkedList< Constant >( );
				this.categories.put( cat , cContents );
			}
			cContents.add( c );
		}
	}


	/**
	 * Constants registration method.
	 * 
	 * This method is responsible for updating the in-base constants data when new definitions are
	 * registered. It will add new constants or update existing one. Finally, if some of the updated
	 * constants are used by any constants user instance, it will send update notifications.
	 * 
	 * @param definitions
	 *            the constants to register
	 */
	synchronized void registerConstants( Collection< ConstantDefinition > definitions )
	{
		Set< String > toNotify = new HashSet< String >( );
		final List< ConstantDefinition > toUpdate = new LinkedList< ConstantDefinition >( );

		for ( ConstantDefinition def : definitions ) {
			// Handle new definitions
			Constant constant = this.registeredConstants.get( def.name );
			if ( constant == null ) {
				toUpdate.add( def );
				toNotify.add( def.name );
				this.sysLog.log( LogLevel.DEBUG , "registering constant " + def.name );
				continue;
			}

			boolean sameDescription = this.compareConstantDescription( constant , def );
			boolean sameBounds = this.compareConstantBounds( constant , def );
			if ( sameDescription && sameBounds ) {
				continue;
			}

			// Make sure the value is correct if the bounds have been changed
			if ( !sameBounds && this.updateValue( constant , def ) ) {
				toNotify.add( def.name );
			}

			toUpdate.add( def );
			this.sysLog.log( LogLevel.DEBUG , "updating definition of constant " + def.name );
		}

		// Apply updates and notify users
		if ( !toUpdate.isEmpty( ) ) {
			this.sysLog.flush( );
			this.tTemplate.execute( new TransactionCallbackWithoutResult( ) {

				@Override
				protected void doInTransactionWithoutResult( TransactionStatus status )
				{
					updateConstants( toUpdate );
					loadConstants( );
				}

			} );

			if ( !toNotify.isEmpty( ) ) {
				this.notify( toNotify );
			}
		}
	}


	/**
	 * Constants user registration
	 * 
	 * This method registers (or re-registers) a constants user, indicating the constants used by
	 * the instance and for which update notifications should be sent. Once the user has been
	 * registered, an attempt at sending the initial notification is made.
	 * 
	 * @param user
	 *            the constants user to register
	 * @param constants
	 *            the names of the constants for which notifications are required
	 */
	synchronized void registerUser( ConstantsUser user , Set< String > constants )
	{
		// If this user was already registered, unregister it
		if ( this.userConstants.containsKey( user ) ) {
			this.unregisterUser( user );
		}

		// Add user <-> constant maps
		this.userConstants.put( user , new HashSet< String >( constants ) );
		for ( String cn : constants ) {
			Set< ConstantsUser > users;
			users = this.constantUsers.get( cn );
			if ( users == null ) {
				users = new HashSet< ConstantsUser >( );
				this.constantUsers.put( cn , users );
			}
			users.add( user );
		}

		// Try sending the user its initial notification
		this.attemptFirstNotification( user );
	}


	/**
	 * Unregisters a constants user.
	 * 
	 * This method removes a constants user from the manager, preventing further notifications to be
	 * sent.
	 * 
	 * @param user
	 *            the constants user to unregister
	 */
	synchronized void unregisterUser( ConstantsUser user )
	{
		Set< String > constants = this.userConstants.get( user );
		if ( constants == null ) {
			return;
		}

		this.userConstants.remove( user );
		this.notifiedUsers.remove( user );
		for ( String constant : constants ) {
			Set< ConstantsUser > users = this.constantUsers.get( constant );
			users.remove( user );
			if ( users.isEmpty( ) ) {
				this.constantUsers.remove( constant );
			}
		}
	}


	/**
	 * Gets the names of the constant categories.
	 * 
	 * This method will make sure the constant categories map is up-to-date, then return a copy of
	 * its keys.
	 * 
	 * @return the names of all constant categories
	 */
	synchronized Set< String > getCategories( )
	{
		this.updateCategories( );
		return new HashSet< String >( this.categories.keySet( ) );
	}


	/**
	 * Accesses the constant data instances from a category.
	 * 
	 * This method will make sure the constant categories map is up-to-date, then return a copy of
	 * the list corresponding to the specified category. If the category is not present in the map,
	 * <code>null</code> will be returned.
	 * 
	 * @param cat
	 *            the category's name
	 * @return the list of constant data instances, or <code>null</code> if the category is
	 *         undefined
	 */
	synchronized List< Constant > getCategory( String cat )
	{
		this.updateCategories( );
		List< Constant > listCst = this.categories.get( cat );
		if ( listCst != null ) {
			return new LinkedList< Constant >( listCst );
		}
		return null;
	}


	/**
	 * Modifies the value of a single constant.
	 * 
	 * This method is called by administrative sessions in order to modify a single constant. It
	 * checks that the constant exists and that its new value is valid, then updates the database
	 * entry. Once this is done, it notifies the constant's users and returns the previous value.
	 * 
	 * @param name
	 *            the name of the constant to modify
	 * @param value
	 *            the constant's new value
	 * @return the constant's previous value
	 * @throws UnknownConstantError
	 *             if the specified constant does not exist
	 * @throws InvalidConstantValue
	 *             if the specified value is out of bounds
	 */
	synchronized Double setValue( final String name , final Double value , final int admin )
			throws UnknownConstantError , InvalidConstantValue
	{
		// Check name
		Constant c = this.registeredConstants.get( name );
		if ( c == null ) {
			throw new UnknownConstantError( name );
		}

		// Check value
		if ( c.getMinValue( ) != null && value < c.getMinValue( ) || c.getMaxValue( ) != null
				&& value > c.getMaxValue( ) ) {
			throw new InvalidConstantValue( );
		}

		// Store constant
		try {
			this.tTemplate.execute( new TransactionCallbackWithoutResult( ) {

				@Override
				protected void doInTransactionWithoutResult( TransactionStatus status )
				{
					fSetConstant.execute( name , value.floatValue( ) , admin );
				}

			} );
		} catch ( RuntimeException t ) {
			this.sysLog.log( LogLevel.ERROR , "could not update constant " + c.getName( ) , t ).flush( );
			throw t;
		}

		// Update in-memory copy
		Double old = c.getValue( );
		c.setValue( value );

		// Notify constant users
		this.notify( c.getName( ) );

		return old;
	}
}
