package com.deepclone.lw.beans.sys;


import java.net.InetAddress;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.deepclone.lw.cmd.admin.users.SessionTerminationType;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.interfaces.session.SessionTypeDefiner;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.session.SessionCommandException;
import com.deepclone.lw.session.SessionIdentifierException;
import com.deepclone.lw.session.SessionReference;
import com.deepclone.lw.session.SessionStateException;
import com.deepclone.lw.utils.RandomStringGenerator;



/**
 * Session data.
 * 
 * @author tseeker
 */
class ServerSessionData
		implements ServerSession
{

	/** Data store */
	private Map< String , Object > storage = new HashMap< String , Object >( );

	/** Session identifier */
	private String identifier;

	/** Definer for the session's type */
	private SessionTypeDefiner definer;

	/** Authentication challenge generator */
	private RandomStringGenerator challengeGenerator;

	/** Last authentication challenge */
	private String lastChallenge;

	/** Session expiration date */
	private Date expire;

	/** Termination reason */
	SessionTerminationType termination = null;

	/** IPv4 address of the client */
	private InetAddress address;

	/** Client type identifier */
	private String client;


	/**
	 * Initialises a new session.
	 * 
	 * @param generator
	 *            authentication challenge generator
	 * @param definer
	 *            session type definer
	 * @param client
	 *            type of the client interface initiating the session
	 * @param address
	 *            IP address the session is being initiated from
	 */
	public ServerSessionData( RandomStringGenerator generator , SessionTypeDefiner definer , String client ,
			InetAddress address )
	{
		this.challengeGenerator = generator;
		this.definer = definer;
		this.expire = new Date( new Date( ).getTime( ) + 5000L );
		this.definer.initialise( this );
		this.client = client;
		this.address = address;
	}


	/**
	 * Sets the session's identifier
	 * 
	 * @param id
	 *            the session's identifier
	 */
	public void setIdentifier( String id )
	{
		this.identifier = id;
	}


	/* Documented in ServerSession interface */
	@Override
	public String getIdentifier( )
	{
		return this.identifier;
	}


	/* Documented in ServerSession interface */
	@Override
	public String getClient( )
	{
		return this.client;
	}


	/* Documented in ServerSession interface */
	@Override
	public InetAddress getAddress( )
	{
		return this.address;
	}


	/* Documented in ServerSession interface */
	@Override
	public String getChallenge( )
	{
		return this.lastChallenge;
	}


	/* Documented in ServerSession interface */
	@Override
	public void setExpirationDate( Date date )
	{
		this.expire = date;
	}


	/** @return the session's expiration date */
	public Date getExpirationDate( )
	{
		return this.expire;
	}


	/**
	 * Sets the termination type and handles termination
	 * 
	 * @param termination
	 *            type of session termination
	 */
	synchronized public void handleTermination( SessionTerminationType termination )
	{
		if ( this.termination == null ) {
			this.termination = termination;
			this.definer.terminate( this , termination );
		}
	}


	/* Documented in ServerSession interface */
	@Override
	public SessionTerminationType getTerminationType( )
	{
		return this.termination;
	}


	/* Documented in ServerSession interface */
	@Override
	public void terminate( )
	{
		this.handleTermination( SessionTerminationType.MANUAL );
	}


	@Override
	public void terminate( SessionTerminationType reason )
	{
		this.handleTermination( reason );
	}


	/* Documented in ServerSession interface */
	@Override
	public void put( String key , Object value )
	{
		if ( value == null ) {
			this.storage.remove( key );
		} else {
			this.storage.put( key , value );
		}
	}


	/* Documented in ServerSession interface */
	@Override
	public < T > T get( String key , Class< T > type )
	{
		Object obj = this.storage.get( key );
		try {
			return ( obj == null ) ? null : type.cast( obj );
		} catch ( ClassCastException e ) {
			return null;
		}
	}


	/**
	 * Generates a {@link SessionReference} from the session's current state.
	 * 
	 * @return the new session reference
	 */
	synchronized public SessionReference toSessionReference( )
	{
		if ( this.termination != null ) {
			return null;
		}

		boolean auth = this.definer.isAuthenticated( this );
		String extra;
		if ( auth ) {
			extra = this.definer.getState( this );
		} else {
			extra = this.challengeGenerator.generate( );
			this.lastChallenge = extra;
		}

		return new SessionReference( this.identifier , this.definer.getName( ) , auth , extra );
	}


	/** Attempts to authenticate the session */
	synchronized public void authenticate( String identifier , String sha1Hash , String md5Hash )
			throws SessionStateException , SessionIdentifierException
	{
		if ( this.termination != null ) {
			throw new SessionIdentifierException( this.identifier );
		}
		this.definer.authenticate( this , identifier , sha1Hash , md5Hash );
	}


	/**
	 * Attempts to execute a command on the session
	 */
	synchronized public CommandResponse execute( Command command )
			throws SessionStateException , SessionCommandException , SessionIdentifierException
	{
		if ( this.termination != null ) {
			throw new SessionIdentifierException( this.identifier );
		}
		return this.definer.execute( this , command );
	}
}
