package com.deepclone.lw.beans.sys;


import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import com.deepclone.lw.interfaces.eventlog.SystemLogger;
import com.deepclone.lw.interfaces.sys.ConstantsUser;
import com.deepclone.lw.interfaces.sys.Ticker;



/**
 * Main ticker control thread.
 * 
 * <p>
 * This class implements the ticker bean's main control thread, executed through a Java
 * {@link Timer} instance. It supports changing the timer's period when the <em>ticker.interval</em>
 * constant is modified, and includes the code that pauses or reactivates the ticker.
 * 
 * @author tseeker
 */
class TickerThread
		implements ConstantsUser
{
	private static class TaskStat
	{
		public final int id;
		public final TickerTask task;
		public final int modulo;
		public int counter = 0;


		public TaskStat( int id , TickerTask task , int modulo )
		{
			this.id = id;
			this.task = task;
			this.task.start( );
			this.modulo = modulo;
		}

	}

	private Timer timer;

	private long interval = 0;

	private boolean paused = false;

	private int taskIndex = 0;

	private int nTasks = 0;

	private TaskStat[] tasks;

	private TickerTaskStatusHandler handler;

	private SystemLogger logger;


	TickerThread( SystemLogger logger , TickerTaskStatusHandler handler )
	{
		this.timer = null;
		this.tasks = new TaskStat[ 5 ];
		this.handler = handler;
		this.logger = logger;
	}


	synchronized private void runTasks( )
	{
		if ( this.paused ) {
			return;
		}
		this.taskIndex = ( this.taskIndex + 1 ) % this.tasks.length;

		TaskStat toExecute = this.tasks[ this.taskIndex ];
		if ( toExecute == null || !this.handler.isTaskRunning( toExecute.id ) ) {
			return;
		}

		toExecute.counter = ( toExecute.counter + 1 ) % toExecute.modulo;
		if ( toExecute.counter != 0 ) {
			return;
		}

		if ( toExecute.task.isFinished( ) ) {
			this.tasks[ this.taskIndex ] = null;
			this.nTasks--;
			return;
		}

		toExecute.task.startTask( );
	}


	/**
	 * Updates the ticker's frequency.
	 * 
	 * <p>
	 * When the <em>ticker.interval</em> constant is changed, this method will cancel the current
	 * scheduling and re-start it using the new interval.
	 */
	@Override
	synchronized public void setConstants( boolean initial , Map< String , Double > values )
	{
		this.interval = Math.round( values.get( "ticker.interval" ) );
		this.reschedule( );
	}


	private void reschedule( )
	{
		if ( this.interval == 0 ) {
			return;
		}

		if ( this.timer != null ) {
			this.timer.cancel( );
		}

		TimerTask task = new TimerTask( ) {
			@Override
			public void run( )
			{
				runTasks( );
			}
		};

		this.timer = new Timer( "Main ticker thread" );
		this.timer.scheduleAtFixedRate( task , 1 , this.interval / this.tasks.length );
	}


	synchronized void registerTask( int id , Ticker.Frequency frequency , String name , Runnable task )
	{
		if ( this.nTasks == this.tasks.length ) {
			TaskStat[] nState = new TaskStat[ this.tasks.length * 2 ];
			for ( int i = 0 ; i < this.tasks.length ; i++ ) {
				nState[ i * 2 ] = this.tasks[ i ];
			}
			this.taskIndex *= 2;
			this.tasks = nState;
			this.reschedule( );
		}

		int modulo;
		switch ( frequency ) {
			case HIGH:
				modulo = 1;
				break;
			case LOW:
				modulo = 30;
				break;
			case MEDIUM:
				modulo = 6;
				break;
			case MINUTE:
				modulo = 12;
				break;
			default:
				throw new RuntimeException( "Invalid timer frequency " + frequency );
		}

		for ( int i = 0 ; i < this.tasks.length ; i++ ) {
			if ( this.tasks[ i ] == null ) {
				this.tasks[ i ] = new TaskStat( id , new TickerTask( name , task , this.logger ) , modulo );
				break;
			}
		}
		this.nTasks ++;
	}


	/**
	 * Pauses the ticker.
	 * 
	 * <p>
	 * This method implements the bean's pause method. If the ticker is not already paused, it will
	 * set the {@link #paused} flag, then wait for all running tasks to complete.
	 * 
	 * @throws IllegalStateException
	 *             if the ticker is already paused.
	 */
	synchronized void pause( )
			throws IllegalStateException
	{
		if ( this.paused ) {
			throw new IllegalStateException( "already paused" );
		}
		this.paused = true;
		for ( int i = 0 ; i < this.tasks.length ; i++ ) {
			TaskStat ts = this.tasks[ i ];
			if ( ts == null ) {
				continue;
			}
			if ( ts.task.isFinished( ) ) {
				this.tasks[ i ] = null;
				this.nTasks--;
				continue;
			}
			ts.task.waitForTask( );
		}
	}


	/**
	 * Restarts the ticker after it's been paused.
	 * 
	 * @throws IllegalStateException
	 *             if the ticker was not paused.
	 */
	synchronized void unpause( )
			throws IllegalStateException
	{
		if ( !this.paused ) {
			throw new IllegalStateException( "not paused" );
		}
		this.paused = false;
	}


	/**
	 * Terminates all tasks in all task sets, then stops the timer.
	 */
	synchronized void terminate( )
	{
		for ( TaskStat ts : this.tasks ) {
			if ( ts == null ) {
				continue;
			}
			ts.task.terminate( );
		}
		this.nTasks = 0;
		this.tasks = new TaskStat[ 5 ];
		if ( this.timer != null ) {
			this.timer.cancel( );
		}
	}


	/**
	 * @return <code>true</code> the ticker is currently running or <code>false</code> if it has
	 *         been paused.
	 */
	synchronized boolean isActive( )
	{
		return !this.paused;
	}
}
