/*
 * Check the variant of verse.create_resource_providers( ) which takes a
 * resource statistics record as a parameter.
 */
BEGIN;

	/* Temporary table which will store information about calls to the
	 * verse.create_resource_provider( ) function
	 */
	CREATE TEMPORARY TABLE test_result(
		planet		INT ,
		resource	INT ,
		quantity	DOUBLE PRECISION ,
		difficulty	DOUBLE PRECISION ,
		recovery	DOUBLE PRECISION
	) ON COMMIT DROP;

	/* Replace verse.list_random_planets_in() with a function that simply
	 * counts.
	 */
	CREATE OR REPLACE FUNCTION verse.list_random_planets_in(
				_area verse.generator_area_type ,
				_count INT )
			RETURNS SETOF INT
			STRICT VOLATILE
			SECURITY INVOKER
		AS $list_random_planets_in$
	DECLARE
		i INT;
	BEGIN
		FOR i IN 1 .. _count
		LOOP
			RETURN NEXT i;
		END LOOP;
	END;
	$list_random_planets_in$ LANGUAGE PLPGSQL;
	
	/* Replace verse.create_resource_provider( ) with a function that writes
	 * to the test_result temporary table.
	 */
	CREATE OR REPLACE FUNCTION verse.create_resource_provider(
			_planet					INT ,
			_data					verse.resprov_generator_type ,
			INOUT _providers_left	INT ,
			INOUT _tot_quantity		DOUBLE PRECISION ,
			INOUT _tot_difficulty	DOUBLE PRECISION ,
			INOUT _tot_recovery		DOUBLE PRECISION )
		STRICT VOLATILE
		SECURITY INVOKER
	AS $create_resource_provider$
	BEGIN
		INSERT INTO test_result VALUES(
			_planet , _data.resource_name_id ,
			_tot_quantity / _providers_left::DOUBLE PRECISION ,
			_tot_difficulty / _providers_left::DOUBLE PRECISION ,
			_tot_recovery / _providers_left::DOUBLE PRECISION
		);
		
		_tot_quantity := ( _providers_left - 1 ) * _tot_quantity / _providers_left;
		_tot_difficulty := ( _providers_left - 1 ) * _tot_difficulty / _providers_left;
		_tot_recovery := ( _providers_left - 1 ) * _tot_recovery / _providers_left;
		_providers_left := _providers_left - 1;
	END;
	$create_resource_provider$ LANGUAGE PLPGSQL;

	/***** TESTS BEGIN HERE *****/
	SELECT plan( 11 );


	/* First set of test: empty universe */
	SELECT verse.create_resource_providers(
		ROW( 0 , 0 , 1 , 1 ) ,
		ROW( 42 , 0.0 , 0.0 , 0.5 , 0.0 , 5.0 , 0.0 , 0.0 , 0.75 , 0.0 , 0.0 , 0.5 , 0 )
	);

	SELECT diag_test_name( 'verse.create_resource_provider( area , stats ) - Empty universe - Providers created' );
	SELECT is( COUNT(*)::INT , 10 ) FROM test_result WHERE resource = 42;
	SELECT diag_test_name( 'verse.create_resource_provider( area , stats ) - Empty universe - Planet identifiers' );
	SELECT results_eq(
		'SELECT planet FROM test_result ORDER BY planet' ,
		ARRAY[ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 ] 
	);	

	SELECT diag_test_name( 'verse.create_resource_provider( area , stats ) - Empty universe - Total quantity' );
	SELECT is( SUM(quantity)::NUMERIC , 50.0 ) FROM test_result;
	SELECT diag_test_name( 'verse.create_resource_provider( area , stats ) - Empty universe - Quantity per planet' );
	SELECT is( COUNT(*)::INT , 10 ) FROM test_result
		WHERE quantity = 5;

	SELECT diag_test_name( 'verse.create_resource_provider( area , stats ) - Empty universe - Total difficulty' );
	SELECT is( SUM(difficulty)::NUMERIC , 7.5 ) FROM test_result;
	SELECT diag_test_name( 'verse.create_resource_provider( area , stats ) - Empty universe - Difficulty per planet' );
	SELECT is( COUNT(*)::INT , 10 ) FROM test_result
		WHERE difficulty = 0.75;

	SELECT diag_test_name( 'verse.create_resource_provider( area , stats ) - Empty universe - Total recovery rate' );
	SELECT is( SUM(recovery)::NUMERIC , 5.0 ) FROM test_result;
	SELECT diag_test_name( 'verse.create_resource_provider( area , stats ) - Empty universe - Recovery rate per planet' );
	SELECT is( COUNT(*)::INT , 10 ) FROM test_result
		WHERE recovery = 0.5;


	/* Second set of tests: balancing providers presence */
	SELECT diag_test_name( 'verse.create_resource_provider( area , stats ) - Providers created in balanced universe' );
	DELETE FROM test_result;
	SELECT verse.create_resource_providers(
		ROW( 0 , 0 , 1 , 1 ) ,
		ROW( 42 , 20.0 , 10.0 , 0.5 , 0.0 , 5.0 , 0.0 , 0.0 , 0.75 , 0.0 , 0.0 , 0.5 , 0 )
	);
	SELECT is( COUNT(*)::INT , 10 ) FROM test_result WHERE resource = 42;

	SELECT diag_test_name( 'verse.create_resource_provider( area , stats ) - Providers created in universe with low providers count' );
	DELETE FROM test_result;
	SELECT verse.create_resource_providers(
		ROW( 0 , 0 , 1 , 1 ) ,
		ROW( 42 , 20.0 , 5.0 , 0.5 , 0.0 , 5.0 , 0.0 , 0.0 , 0.75 , 0.0 , 0.0 , 0.5 , 0 )
	);
	SELECT is( COUNT(*)::INT , 15 ) FROM test_result WHERE resource = 42;

	SELECT diag_test_name( 'verse.create_resource_provider( area , stats ) - Providers created in universe with high providers count' );
	DELETE FROM test_result;
	SELECT verse.create_resource_providers(
		ROW( 0 , 0 , 1 , 1 ) ,
		ROW( 42 , 20.0 , 15.0 , 0.5 , 0.0 , 5.0 , 0.0 , 0.0 , 0.75 , 0.0 , 0.0 , 0.5 , 0 )
	);
	SELECT is( COUNT(*)::INT , 5 ) FROM test_result WHERE resource = 42;

	SELECT * FROM finish( );
ROLLBACK;