package com.deepclone.lw.interfaces.game.resources;


/**
 * Data access object for mining settings
 * 
 * <p>
 * This interface provides methods which call stored procedures that control mining settings for
 * both empires and planets.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public interface MiningSettingsDAO
{

	/**
	 * Start an empire settings update
	 * 
	 * <p>
	 * Start an empire settings update by calling the <code>emp.mset_update_start(INT)</code> stored
	 * procedure.
	 * 
	 * @param empire
	 *            the identifier of the empire whose settings will be updated
	 * 
	 * @return <code>true</code> on success, <code>false</code> on failure
	 */
	public boolean startUpdate( int empire );


	/**
	 * Start a planet settings update
	 * 
	 * <p>
	 * Start a planet settings update by calling the <code>emp.mset_update_start(INT,INT)</code>
	 * stored procedure.
	 * 
	 * @param empire
	 *            the identifier of the empire who supposedly owns the planet
	 * @param planet
	 *            the identifier of the planet whose settings will be changed
	 * 
	 * @return <code>true</code> if the planet belongs to the specified empire and has specific
	 *         mining settings
	 */
	public boolean startUpdate( int empire , int planet );


	/**
	 * Set the new extraction priority of a resource
	 * 
	 * <p>
	 * Call the <code>emp.mset_update_set(TEXT,INT)</code> stored procedure to set the new
	 * extraction priority for some type of resources. An mining settings update (empire- or
	 * planet-wide) must have been initiated before this method is called.
	 * 
	 * @param resource
	 *            the identifier of the resource
	 * @param priority
	 *            the new extraction priority for the resource
	 * 
	 * @return <code>true</code> on success, <code>false</code> if the resource did not exist.
	 */
	public boolean setNewPriority( String resource , int priority );


	/**
	 * Apply the current settings update
	 * 
	 * <p>
	 * This method applies a previously initiated mining settings update by calling the
	 * <code>emp.mset_update_apply()</code> stored procedure.
	 * 
	 * @return <code>true</code> on success, <code>false</code> if one of the priorities was
	 *         invalid.
	 */
	public boolean applyUpdate( );


	/**
	 * Toggle the source of a planet's mining settings
	 * 
	 * <p>
	 * Call the <code>emp.mset_toggle_source(INT,INT)</code> stored procedure to toggle the source
	 * of a planet's mining priorities.
	 * 
	 * @param empire
	 *            the empire's identifier
	 * @param planet
	 *            the planet's identifier
	 */
	public void togglePlanet( int empire , int planet );
}
