package com.deepclone.lw.interfaces.game.updates;


/**
 * Game update processor
 * 
 * <p>
 * The game update processor allows game update cycles to be executed. It also provides the ability
 * to terminated an existing update cycle.
 * 
 * <p>
 * The game update processor includes a locking mechanism which can be used to prevent two
 * components (for example the game update ticker task and the administration interface) from
 * executing game updates simultaneously.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public interface GameUpdateProcessor
{

	/**
	 * Try locking the game update processor
	 * 
	 * <p>
	 * Test and set the processor's lock. This lock is not re-entrant and should only be called once
	 * per locking thread.
	 * 
	 * @return <code>true</code> if the lock was acquired, <code>false</code> it some other
	 *         component had already locked the processor.
	 */
	public boolean tryLock( );


	/**
	 * Unlock the game update processor
	 * 
	 * <p>
	 * Reset the processor's lock. This method should be called from the thread which actually
	 * locked the processor in the first place, although that will not be checked.
	 * 
	 * @throws IllegalStateException
	 *             if the processor is not locked.
	 */
	public void unlock( )
			throws IllegalStateException;


	/**
	 * End the previous update cycle
	 * 
	 * <p>
	 * Check if an update cycle had already started. If such is the case, execute all remaining
	 * update batches.
	 * 
	 * @return <code>true</code> if an update cycle was processed or if the system is under
	 *         maintenance, <code>false</code> otherwise.
	 */
	public boolean endPreviousCycle( );


	/**
	 * Execute a full update cycle
	 * 
	 * <p>
	 * Check for an update cycle that was already started. If one exists, finish it. Otherwise,
	 * start a new cycle and execute all batches in separate transactions.
	 * 
	 * <p>
	 * Update cycles will not be processed if the system is under maintenance. In addition, if the
	 * system enters maintenance mode while updates are being processed, the processing will stop
	 * after the current batch.
	 */
	public void executeUpdateCycle( );

}