package com.deepclone.lw.interfaces.i18n;


import java.util.Map;
import java.util.Set;



/**
 * Translations database administration session
 * 
 * This interface defines the various methods allowing an administrator to make changes to the
 * translations database.
 * 
 * @author tseeker
 */
public interface I18NAdministration
{

	/**
	 * @return the set of all languages identifiers, including languages that are not 100% supported
	 */
	public Set< String > getLanguages( );


	/**
	 * Obtains the name of a language
	 * 
	 * @param language
	 *            the language's identifier
	 * @return the language's name
	 * @throws UnknownLanguageException
	 *             if the language does not exist
	 */
	public String getLanguageName( String language )
			throws UnknownLanguageException;


	/**
	 * Returns the ratio of translated strings in a language relative to the total amount of string
	 * definitions.
	 * 
	 * @param language
	 *            the language's identifier
	 * @return the language's support ratio
	 * @throws UnknownLanguageException
	 *             if the language does not exist
	 */
	public double getLanguageSupport( String language )
			throws UnknownLanguageException;


	/**
	 * Adds a new, totally unsupported language.
	 * 
	 * @param language
	 *            the new language's identifier
	 * @param name
	 *            the name of the new language
	 * @throws DuplicateLanguageException
	 *             if the language identifier is already in use
	 */
	public void createLanguage( String language , String name )
			throws DuplicateLanguageException;


	/**
	 * Updates a language's name
	 * 
	 * @param language
	 *            the language's identifier
	 * @param name
	 *            the new name of the language
	 * @throws UnknownLanguageException
	 *             if the language does not exist
	 */
	public void setLanguageName( String language , String name )
			throws UnknownLanguageException;


	/**
	 * @return the set of all string identifiers
	 */
	public Set< String > getStrings( );


	/**
	 * Accesses a string's translation.
	 * 
	 * @param language
	 *            the language identifier
	 * @param string
	 *            the string identifier
	 * @return the translation for the specified string in the specified language, or null if the
	 *         string has not been translated in this language
	 * @throws UnknownStringException
	 *             if the string identifier is invalid
	 * @throws UnknownLanguageException
	 *             if the language does not exist
	 */
	public String getTranslation( String language , String string )
			throws UnknownStringException , UnknownLanguageException;


	/**
	 * Modifies or creates a string's translation in a given language
	 * 
	 * @param language
	 *            the language identifier
	 * @param string
	 *            the string identifier
	 * @param translation
	 *            the new translated string
	 * @return true if the string was created, false if it was updated
	 * @throws UnknownStringException
	 *             if the string identifier is invalid
	 * @throws UnknownLanguageException
	 *             if the language does not exist
	 */
	public boolean updateTranslation( String language , String string , String translation )
			throws UnknownStringException , UnknownLanguageException;


	/**
	 * Creates a new string and adds some of its translations. For this method to work as expected,
	 * no 100% supported language must become unsupported.
	 * 
	 * @param string
	 *            the new string's identifier
	 * @param translations
	 *            a map whose keys are language identifiers and whose values are translations.
	 * @throws DuplicateStringException
	 *             if the new string's identifier already exists
	 * @throws UnknownLanguageException
	 *             if one of the languages does not exist
	 * @throws InvalidUpdateException
	 *             if the creation of the string would cause one or more of the currently supported
	 *             languages to become unsupported
	 */
	public void createString( String string , Map< String , String > translations )
			throws DuplicateStringException , UnknownLanguageException , InvalidUpdateException;

}
