package com.deepclone.lw.interfaces.session;


import com.deepclone.lw.cmd.admin.users.SessionTerminationType;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.session.SessionCommandException;
import com.deepclone.lw.session.SessionStateException;



/**
 * Interface for session type definers.
 * 
 * <p>
 * This interface defines methods which allow sessions to be managed.
 * 
 * @author tseeker
 */
public interface SessionTypeDefiner
{

	/** @return the name of the session type */
	public String getName( );


	/**
	 * Initialises a session.
	 * 
	 * <p>
	 * This method should initialise the specified session, setting its initial data and expiration
	 * date. When the method is called by the {@link SessionManager}, the session has no identifier
	 * - it is only assigned once initialisation has completed.
	 * 
	 * @param session
	 *            the session to initialise
	 */
	public void initialise( ServerSession session );


	/**
	 * Determines whether the specified session has undergone authentication.
	 * 
	 * @param session
	 *            the session whose status is to be determined.
	 * @return <code>true</code> if the session has undergone authentication or doesn't need to
	 *         authenticate, <code>false</code> if authentication is required
	 */
	public boolean isAuthenticated( ServerSession session );


	/**
	 * Returns the current state of the session.
	 * 
	 * <p>
	 * This method returns the current state of an authenticated session. It is useful for session
	 * definers that implement multi-state sessions, accepting different sets of commands in each
	 * state.
	 * 
	 * @param session
	 *            the session whose state is to be determined
	 * @return the session's state.
	 */
	public String getState( ServerSession session );


	/**
	 * Authenticates a session.
	 * 
	 * <p>
	 * This method attempts to authenticate a session, modifying the session's data accordingly if
	 * successful.
	 * 
	 * @param session
	 *            the session being authenticated
	 * @param identifier
	 *            authentication identifier (for example an account's mail address)
	 * @param sha1Hash
	 *            SHA-1 hash of the challenge response
	 * @param md5Hash
	 *            MD5 hash of the challenge response
	 * @throws SessionStateException
	 *             if the session type doesn't support authentication or if the session had already
	 *             undergone authentication.
	 */
	public void authenticate( ServerSession session , String identifier , String sha1Hash , String md5Hash )
			throws SessionStateException;


	/**
	 * Executes a command.
	 * 
	 * <p>
	 * This method executes the specified command in the session's context, returning the command's
	 * results.
	 * 
	 * @param session
	 *            the session whose context is to be used when executing the command
	 * @param command
	 *            the command to execute
	 * @return whatever the command execution yields.
	 * @throws SessionStateException
	 *             if the session type requires authentication and the session is not authenticated.
	 * @throws SessionCommandException
	 *             if the session type does not support the specified command.
	 */
	public CommandResponse execute( ServerSession session , Command command )
			throws SessionStateException , SessionCommandException;


	/**
	 * Handles session termination.
	 * 
	 * <p>
	 * This method will execute code specific to a session type when a session is being terminated
	 * (either through the session type definer itself or for other reasons).
	 * 
	 * @param session
	 *            the session that's being terminated.
	 * @param reason
	 *            the reason why the session's being terminated
	 */
	public void terminate( ServerSession session , SessionTerminationType reason );

}
