package com.deepclone.lw.cli.xmlimport;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.evdef.*;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.XStreamException;



/**
 * Event definitions loader
 * 
 * <p>
 * This class implements loading event definitions from XML files. The definitions will be extracted
 * and verified.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class EventDefinitionLoader
{
	/** The file to read the XML from */
	private final File file;


	/**
	 * Initialise the loader
	 * 
	 * @param file
	 *            the XML file that contains the definitions
	 */
	public EventDefinitionLoader( File file )
	{
		this.file = file.getAbsoluteFile( );
	}


	/**
	 * Initialise the necessary XStream instance
	 * 
	 * <p>
	 * Initialise the XStream instance by processing annotations in all event definition importable
	 * data classes.
	 * 
	 * @return the XStream instance to use when loading the data
	 */
	private XStream initXStream( )
	{
		XStream xstream = new XStream( );
		xstream.processAnnotations( EventDefinitions.class );
		xstream.processAnnotations( BooleanEventField.class );
		xstream.processAnnotations( IntegerEventField.class );
		xstream.processAnnotations( RealEventField.class );
		xstream.processAnnotations( TextEventField.class );
		xstream.processAnnotations( I18NEventField.class );
		xstream.processAnnotations( EntityEventField.class );
		return xstream;
	}


	/**
	 * Load the event definitions
	 * 
	 * <p>
	 * Load the XML file and process the definitions file using XStream.
	 * 
	 * @return the top-level importable data instance
	 * 
	 * @throws DataImportException
	 *             if reading from the file or parsing its contents fail
	 */
	private EventDefinitions loadXMLFile( )
			throws DataImportException
	{
		FileInputStream fis;
		try {
			fis = new FileInputStream( this.file );
		} catch ( FileNotFoundException e ) {
			throw new DataImportException( "Unable to load event definitions" , e );
		}

		try {
			try {
				XStream xstream = this.initXStream( );
				return (EventDefinitions) xstream.fromXML( fis );
			} finally {
				fis.close( );
			}
		} catch ( IOException e ) {
			throw new DataImportException( "Input error while loading event definitions" , e );
		} catch ( XStreamException e ) {
			throw new DataImportException( "XML error while loading event definitions" , e );
		}
	}


	/**
	 * Load and process I18N definition
	 * 
	 * <p>
	 * Attempt to load all I18N definitions, make sure they are valid, then set the original file's
	 * path.
	 * 
	 * @return the top-level importable data instance
	 * 
	 * @throws DataImportException
	 *             if loading or verifying the data fails
	 */
	public EventDefinitions load( )
			throws DataImportException
	{
		EventDefinitions events = this.loadXMLFile( );
		events.verifyData( );
		events.setReadFrom( this.file );
		return events;
	}

}
