package com.deepclone.lw.cli.xmlimport;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.resources.NaturalResource;
import com.deepclone.lw.cli.xmlimport.data.resources.Resources;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.XStreamException;



/**
 * Resource Loader
 * 
 * <p>
 * This class can be used to load all resource definitions. It extracts them from the XML file and
 * verifies them.
 * 
 * @author <a href="mailto:tim@mitheren.com">T. Rosser</a>
 */
public class ResourceLoader
{
	/** The file to read the XML from */
	private final File file;


	/**
	 * Initialise the loader
	 * 
	 * @param file
	 *            the XML file that contains the definitions
	 */
	public ResourceLoader( File file )
	{
		this.file = file.getAbsoluteFile( );
	}


	/**
	 * Initialise the necessary XStream instance
	 * 
	 * <p>
	 * Initialise the XStream instance by processing the annotations of all the resource importable
	 * data classes.
	 * 
	 * @return the XStream instance to use when loading the data
	 */
	private XStream initXStream( )
	{
		XStream xstream = new XStream( );
		xstream.processAnnotations( Resources.class );
		xstream.processAnnotations( NaturalResource.class );
		return xstream;
	}


	/**
	 * Load the resource definitions
	 * 
	 * <p>
	 * Load the XML file and process the definitions using XStream.
	 * 
	 * @return the top-level importable data instance
	 * 
	 * @throws DataImportException
	 *             if reading from the file or parsing its contents fail
	 */
	private Resources loadXMLFile( )
			throws DataImportException
	{
		FileInputStream fis;
		try {
			fis = new FileInputStream( this.file );
		} catch ( FileNotFoundException e ) {
			throw new DataImportException( "Unable to load resource definitions" , e );
		}

		try {
			try {
				XStream xstream = this.initXStream( );
				return (Resources) xstream.fromXML( fis );
			} finally {
				fis.close( );
			}
		} catch ( IOException e ) {
			throw new DataImportException( "Input error while loading resource definitions" , e );
		} catch ( XStreamException e ) {
			throw new DataImportException( "XML error while loading resource definitions" , e );
		}
	}


	/**
	 * Load and process resource definitions
	 * 
	 * <p>
	 * Attempt to load all resource definitions, ensure they are valid, then set the original file's
	 * path.
	 * 
	 * @return the top-level importable data instance
	 * 
	 * @throws DataImportException
	 *             if loading or verifying the data fails
	 */
	public Resources load( )
			throws DataImportException
	{
		Resources resources = this.loadXMLFile( );
		resources.verifyData( );
		resources.setReadFrom( this.file );
		return resources;
	}
}
