package com.deepclone.lw.cli.xmlimport;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.techs.Technologies;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.XStreamException;



/**
 * Technology Loader
 * 
 * <p>
 * This class can be used to load all technology definitions. It extracts them from the XML file and
 * verifies them.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TechnologyLoader
{
	/** The file to read the XML from */
	private final File file;


	/**
	 * Initialise the loader
	 * 
	 * @param file
	 *            the XML file that contains the definitions
	 */
	public TechnologyLoader( File file )
	{
		this.file = file.getAbsoluteFile( );
	}


	/**
	 * Initialise the necessary XStream instance
	 * 
	 * <p>
	 * Initialise the XStream instance by processing the annotations of all the technology
	 * importable data classes.
	 * 
	 * @return the XStream instance to use when loading the data
	 */
	private XStream initXStream( )
	{
		XStream xstream = new XStream( );
		xstream.processAnnotations( Technologies.class );
		return xstream;
	}


	/**
	 * Load the technology definitions
	 * 
	 * <p>
	 * Load the XML file and process the definitions using XStream.
	 * 
	 * @return the top-level importable data instance
	 * 
	 * @throws DataImportException
	 *             if reading from the file or parsing its contents fail
	 */
	private Technologies loadXMLFile( )
			throws DataImportException
	{
		FileInputStream fis;
		try {
			fis = new FileInputStream( this.file );
		} catch ( FileNotFoundException e ) {
			throw new DataImportException( "Unable to load technology definitions" , e );
		}

		try {
			try {
				XStream xstream = this.initXStream( );
				return (Technologies) xstream.fromXML( fis );
			} finally {
				fis.close( );
			}
		} catch ( IOException e ) {
			throw new DataImportException( "Input error while loading technology definitions" , e );
		} catch ( XStreamException e ) {
			throw new DataImportException( "XML error while loading technology definitions" , e );
		}
	}


	/**
	 * Load and process technology definitions
	 * 
	 * <p>
	 * Attempt to load all technology definitions, ensure they are valid, then set the original
	 * file's path.
	 * 
	 * @return the top-level importable data instance
	 * 
	 * @throws DataImportException
	 *             if loading or verifying the data fails
	 */
	public Technologies load( )
			throws DataImportException
	{
		Technologies techs = this.loadXMLFile( );
		techs.verifyData( );
		techs.setReadFrom( this.file );
		return techs;
	}
}
