package com.deepclone.lw.cli.xmlimport.data.evdef;


import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.ImportableData;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;



/**
 * Event definition
 * 
 * <p>
 * This class represents an event definition entry from the data file. An event definition consists
 * in a few properties, and a potentially empty list of event field definitions.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
@SuppressWarnings( "serial" )
public class EventDefinition
		extends ImportableData
		implements Iterable< EventFieldBase >
{

	/** The event type's identifier */
	@XStreamAsAttribute
	@XStreamAlias( "id" )
	private String identifier;

	/** The default priority */
	@XStreamAsAttribute
	private Integer priority;

	/** Whether users can override the events' priority */
	@XStreamAsAttribute
	private Boolean adjustable;

	/** Prefix of the name and template strings */
	@XStreamAsAttribute
	@XStreamAlias( "i18n-strings" )
	private String i18nStrings;

	/** The list of fields */
	@XStreamImplicit
	private List< EventFieldBase > fields = new ArrayList< EventFieldBase >( );


	/**
	 * Gets the identifier.
	 * 
	 * @return the identifier
	 */
	public String getIdentifier( )
	{
		return this.identifier;
	}


	/**
	 * Gets the priority.
	 * 
	 * @return the priority
	 */
	public int getPriority( )
	{
		return ( this.priority == null ) ? 2 : this.priority;
	}


	/**
	 * Checks if the priority is adjustable.
	 * 
	 * @return <code>true</code> if the priority is adjustable
	 */
	public boolean isAdjustable( )
	{
		return ( this.adjustable == null ) ? true : this.adjustable;
	}


	/**
	 * Gets the prefix of the name and template strings.
	 * 
	 * @return the prefix of the name and template strings.
	 */
	public String getI18NStrings( )
	{
		return this.i18nStrings;
	}


	/**
	 * Allows iteration over the event definition's fields
	 * 
	 * @return an iterator on the event definition's fields
	 */
	@Override
	public Iterator< EventFieldBase > iterator( )
	{
		return Collections.unmodifiableList( this.fields ).iterator( );
	}


	/**
	 * Verify fields data
	 * 
	 * <p>
	 * If no fields are defined, make sure the list is not <code>null</code>; otherwise iterate over
	 * the fields to call their own data checks methods.
	 */
	@Override
	public void verifyData( )
			throws DataImportException
	{
		super.verifyData( );
		if ( this.fields == null ) {
			this.fields = new ArrayList< EventFieldBase >( );
		} else {
			for ( EventFieldBase f : this.fields ) {
				f.verifyData( );
			}
		}
	}

}
