package com.deepclone.lw.cli.xmlimport.data.i18n;


import java.io.File;
import java.io.Serializable;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.ImportableData;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamImplicit;



/**
 * I18N text data
 * 
 * <p>
 * This class represents the contents of the I18N text definitions file. It contains a list of
 * language definitions.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@SuppressWarnings( "serial" )
@XStreamAlias( "lw-text-data" )
public class I18NText
		extends ImportableData
		implements Serializable , Iterable< LanguageDefinition >
{
	/** All present language definitions */
	@XStreamImplicit( itemFieldName = "language" )
	private List< LanguageDefinition > languages = new LinkedList< LanguageDefinition >( );


	/**
	 * Set the source file
	 * 
	 * <p>
	 * Update the definition's own source file, then update all language definitions.
	 */
	@Override
	public void setReadFrom( File readFrom )
	{
		super.setReadFrom( readFrom );
		for ( LanguageDefinition languageDefinition : this.languages ) {
			languageDefinition.setReadFrom( readFrom );
		}
	}


	/**
	 * Verify I18N text data
	 * 
	 * <p>
	 * Check each definition, then make sure both identifiers and language names are unique.
	 */
	@Override
	public void verifyData( )
			throws DataImportException
	{
		if ( this.languages == null || this.languages.isEmpty( ) ) {
			throw new DataImportException( "No language definitions" );
		}

		HashSet< String > identifiers = new HashSet< String >( );
		HashSet< String > names = new HashSet< String >( );
		for ( LanguageDefinition definition : this.languages ) {
			definition.verifyData( );
			this.checkUniqueItem( "identifier" , identifiers , definition.getId( ) );
			this.checkUniqueItem( "name" , names , definition.getName( ) );
		}
	}


	/**
	 * Check that some part of the definition is unique
	 * 
	 * <p>
	 * This helper method is used by {@link #verifyData()} to make sure that names and identifiers
	 * are unique.
	 * 
	 * @param exceptionText
	 *            the name of the item
	 * @param existing
	 *            the set of existing items
	 * @param value
	 *            the item's value
	 * 
	 * @throws DataImportException
	 *             if the item's value is already present in the set of existing items
	 */
	private void checkUniqueItem( String exceptionText , HashSet< String > existing , String value )
			throws DataImportException
	{
		if ( existing.contains( value ) ) {
			throw new DataImportException( "Duplicate language " + exceptionText + " '" + value + "'" );
		}
		existing.add( value );
	}


	/**
	 * Language definition iterator
	 * 
	 * <p>
	 * Grant access to the list of languages in read-only mode.
	 * 
	 * @return a read-only iterator on the list of languages.
	 */
	@Override
	public Iterator< LanguageDefinition > iterator( )
	{
		return Collections.unmodifiableList( this.languages ).iterator( );
	}

}
