package com.deepclone.lw.cli.xmlimport.data.techs;


import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.ImportableData;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamImplicit;



/**
 * Technology definitions data
 * 
 * <p>
 * This class represents the contents of a technology definition XML file. It contains a list of
 * technology definitions.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@SuppressWarnings( "serial" )
@XStreamAlias( "lw-tech-graph" )
public class Technologies
		extends ImportableData
		implements Iterable< Technology >
{
	/** All present technology definitions */
	@XStreamImplicit( itemFieldName = "technology" )
	private final List< Technology > technologies = new LinkedList< Technology >( );


	/**
	 * Check the technology data
	 * 
	 * <p>
	 * Check whether there is a list of technologies, then check each definition to make sure that
	 * it is valid and that its name is unique.
	 */
	@Override
	public void verifyData( )
			throws DataImportException
	{
		if ( this.technologies == null ) {
			throw new DataImportException( "No technology definitions" );
		}

		HashSet< String > names = new HashSet< String >( );
		for ( Technology tech : this.technologies ) {
			tech.verifyData( );
			this.checkUniqueItem( names , tech.getName( ) );
		}
	}


	/**
	 * Checks that the name of a technology is unique
	 * 
	 * <p>
	 * This helper method is used by {@link #verifyData()} to make sure technology names are unique.
	 * 
	 * @param existing
	 *            the existing set of names
	 * @param value
	 *            the name to check
	 * 
	 * @throws DataImportException
	 *             if the name is already present in the set of existing names
	 */
	public void checkUniqueItem( HashSet< String > existing , String value )
			throws DataImportException
	{
		if ( existing.contains( value ) ) {
			throw new DataImportException( "Duplicate technology name '" + value + "'" );
		}
		existing.add( value );
	}


	/**
	 * Technology definition iterator
	 * 
	 * <p>
	 * Grant access to the list of technologies in read-only mode.
	 * 
	 * @return a read-only iterator on the list of technologies.
	 */
	@Override
	public Iterator< Technology > iterator( )
	{
		return Collections.unmodifiableList( this.technologies ).iterator( );
	}

}
