package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.testing.MockResultSet;



/**
 * Tests for the {@link ResourceRowMapper} class.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestResourceRowMapper
{
	/** Planet identifiers found in the "results" */
	private static final int[] PLANET_IDS = new int[] {
			1 , 2 , 3
	};

	/** Resource names found in the three "results", respectively */
	private static final String[] RESOURCE_NAMES = new String[] {
			"Test1" , "Test2" , "Test3"
	};

	/** Income values found in the three results */
	private static final double[] INCOME_VALUES = new double[] {
			4.0 , 5.0 , 6.0
	};

	/** Upkeep values found in the three results */
	private static final double[] UPKEEP_VALUES = new double[] {
			7.0 , 8.0 , 9.0
	};

	/** Resource provider quantity for the second and third rows */
	private static final double[] RP_QUANTITY = {
			10.0 , 11.0
	};

	/** Resource provider capacity for the second and third rows */
	private static final double[] RP_CAPACITY = {
			12.0 , 13.0
	};

	/** Resource provider extraction difficulty for the second and third rows */
	private static final double[] RP_DIFFICULTY = {
			14.0 , 15.0
	};

	/** Resource provider recovery rate for the second and third rows */
	private static final double[] RP_RECOVERY = {
			16.0 , 17.0
	};

	/** Mining priority value used for the third row */
	private static final Integer MINING_PRIORITY = 18;

	/** The fake result set used in the tests */
	private ResultSet resultSet;

	/** The mapper used in the tests */
	private ResourceRowMapper mapper;


	/** Create the mapper and the contents of the fake result set */
	@Before
	public void setUp( )
	{
		this.mapper = new ResourceRowMapper( );

		@SuppressWarnings( "unchecked" )
		HashMap< String , Object > rows[] = new HashMap[ 3 ];
		for ( int i = 0 ; i < 3 ; i++ ) {
			HashMap< String , Object > row = new HashMap< String , Object >( );
			row.put( "planet_id" , PLANET_IDS[ i ] );
			row.put( "resource_name" , RESOURCE_NAMES[ i ] );
			row.put( "pres_income" , INCOME_VALUES[ i ] );
			row.put( "pres_upkeep" , UPKEEP_VALUES[ i ] );
			if ( i > 0 ) {
				row.put( "resprov_quantity_max" , RP_CAPACITY[ i - 1 ] );
				row.put( "resprov_quantity" , RP_QUANTITY[ i - 1 ] );
				row.put( "resprov_difficulty" , RP_DIFFICULTY[ i - 1 ] );
				row.put( "resprov_recovery" , RP_RECOVERY[ i - 1 ] );
				if ( i > 1 ) {
					row.put( "mining_priority" , MINING_PRIORITY );
				}
			}
			rows[ i ] = row;
		}

		this.resultSet = MockResultSet.create( rows );
	}


	/**
	 * Mapping a row with no provider information
	 */
	@Test
	public void testMapNoProvider( )
			throws SQLException
	{
		this.resultSet.absolute( 1 );

		PlanetResourceRow row = this.mapper.mapRow( this.resultSet , 0 );
		assertNotNull( row );
		assertEquals( PLANET_IDS[ 0 ] , row.getPlanetId( ) );
		assertEquals( RESOURCE_NAMES[ 0 ] , row.getDelta( ).getResource( ) );
		assertEquals( INCOME_VALUES[ 0 ] , row.getDelta( ).getIncome( ) , 0 );
		assertEquals( UPKEEP_VALUES[ 0 ] , row.getDelta( ).getUpkeep( ) , 0 );
		assertNull( row.getProvider( ) );
	}


	/**
	 * Mapping a row with a provider information record but no mining priority
	 */
	@Test
	public void testMapWithProvider( )
			throws SQLException
	{
		this.resultSet.absolute( 2 );

		PlanetResourceRow row = this.mapper.mapRow( this.resultSet , 0 );
		assertNotNull( row );
		assertEquals( PLANET_IDS[ 1 ] , row.getPlanetId( ) );
		assertEquals( RESOURCE_NAMES[ 1 ] , row.getDelta( ).getResource( ) );
		assertEquals( INCOME_VALUES[ 1 ] , row.getDelta( ).getIncome( ) , 0 );
		assertEquals( UPKEEP_VALUES[ 1 ] , row.getDelta( ).getUpkeep( ) , 0 );
		assertNotNull( row.getProvider( ) );
		assertEquals( RP_CAPACITY[ 0 ] , row.getProvider( ).getMaximalQuantity( ) , 0 );
		assertEquals( RP_QUANTITY[ 0 ] , row.getProvider( ).getCurrentQuantity( ) , 0 );
		assertEquals( RP_DIFFICULTY[ 0 ] , row.getProvider( ).getDifficulty( ) , 0 );
		assertEquals( RP_RECOVERY[ 0 ] , row.getProvider( ).getRecovery( ) , 0 );
		assertNull( row.getProvider( ).getMiningPriority( ) );
	}


	/**
	 * Mapping a row with a provider information record and a mining priority
	 */
	@Test
	public void testMapWithPriority( )
			throws SQLException
	{
		this.resultSet.absolute( 3 );

		PlanetResourceRow row = this.mapper.mapRow( this.resultSet , 0 );
		assertNotNull( row );
		assertEquals( PLANET_IDS[ 2 ] , row.getPlanetId( ) );
		assertEquals( RESOURCE_NAMES[ 2 ] , row.getDelta( ).getResource( ) );
		assertEquals( INCOME_VALUES[ 2 ] , row.getDelta( ).getIncome( ) , 0 );
		assertEquals( UPKEEP_VALUES[ 2 ] , row.getDelta( ).getUpkeep( ) , 0 );
		assertNotNull( row.getProvider( ) );
		assertEquals( RP_CAPACITY[ 1 ] , row.getProvider( ).getMaximalQuantity( ) , 0 );
		assertEquals( RP_QUANTITY[ 1 ] , row.getProvider( ).getCurrentQuantity( ) , 0 );
		assertEquals( RP_DIFFICULTY[ 1 ] , row.getProvider( ).getDifficulty( ) , 0 );
		assertEquals( RP_RECOVERY[ 1 ] , row.getProvider( ).getRecovery( ) , 0 );
		assertEquals( MINING_PRIORITY , row.getProvider( ).getMiningPriority( ) );
	}
}
