package com.deepclone.lw.beans.bt.es.data;


import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import com.thoughtworks.xstream.XStream;



/**
 * Tests for the {@link ResourceDeltaInformation} XML dump storage class
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestResourceDeltaInformation
{

	/** A string used in tests */
	private static final String TEST_STRING = "This is a test";

	/** A real number used in tests */
	private static final Double TEST_DOUBLE = 4.2;

	/** A record used in the tests */
	private ResourceDeltaInformation rdi;


	/**
	 * Set up the test by creating a new resource delta information record.
	 */
	@Before
	public void setUp( )
	{
		this.rdi = new ResourceDeltaInformation( );
	}


	/**
	 * All fields are set to <code>null</code> by default
	 */
	@Test
	public void testDefaultValues( )
	{
		assertNull( this.rdi.getResource( ) );
		assertNull( this.rdi.getIncome( ) );
		assertNull( this.rdi.getUpkeep( ) );
	}


	/**
	 * Setting and reading the resource's name
	 */
	@Test
	public void testResourceName( )
	{
		this.rdi.setResource( TEST_STRING );
		assertEquals( TEST_STRING , this.rdi.getResource( ) );
	}


	/**
	 * Setting the resource name to <code>null</code> throws {@link InvalidDumpContentsException}
	 */
	@Test
	public void testNullResourceName( )
	{
		try {
			this.rdi.setResource( null );
			fail( "No InvalidDumpContentsException thrown" );
		} catch ( InvalidDumpContentsException exception ) {
			assertEquals( ResourceDeltaInformation.class , exception.getRecordType( ) );
			assertEquals( "resource" , exception.getField( ) );
		}
	}


	/**
	 * Setting and reading the income
	 */
	@Test
	public void testIncome( )
	{
		this.rdi.setIncome( TEST_DOUBLE );
		assertEquals( TEST_DOUBLE , this.rdi.getIncome( ) );
	}


	/**
	 * Setting and reading the upkeep
	 */
	@Test
	public void testUpkeep( )
	{
		this.rdi.setUpkeep( TEST_DOUBLE );
		assertEquals( TEST_DOUBLE , this.rdi.getUpkeep( ) );
	}


	/**
	 * Serialising the instance to XML
	 */
	@Test
	public void testXMLSerialisation( )
	{
		this.rdi.setResource( TEST_STRING );
		this.rdi.setIncome( 0.1 );
		this.rdi.setUpkeep( 0.2 );

		XStream xstream = this.createXStreamInstance( );
		String serialised = xstream.toXML( this.rdi );
		assertNotNull( serialised );
		assertTrue( serialised.startsWith( "<resource-delta " ) );
		assertTrue( serialised.endsWith( "/>" ) );
		assertTrue( serialised.contains( " id=\"" + TEST_STRING + "\"" ) );
		assertTrue( serialised.contains( " income=\"0.1\"" ) );
		assertTrue( serialised.contains( " upkeep=\"0.2\"" ) );
	}


	/**
	 * Deserialising an instance that contains data
	 */
	@Test
	public void testXMLDeserialisation( )
	{
		String xml = "<resource-delta id=\"Test\" income=\"0.1\" upkeep=\"0.2\" />";
		XStream xstream = this.createXStreamInstance( );
		Object deserialised = xstream.fromXML( xml );

		assertNotNull( deserialised );
		assertEquals( ResourceDeltaInformation.class , deserialised.getClass( ) );
		this.rdi = (ResourceDeltaInformation) deserialised;

		assertEquals( "Test" , this.rdi.getResource( ) );
		assertEquals( (Double) 0.1 , this.rdi.getIncome( ) );
		assertEquals( (Double) 0.2 , this.rdi.getUpkeep( ) );
	}


	/**
	 * Deserialising an instance that contains no data
	 */
	@Test
	public void testXMLDeserialisationNoData( )
	{
		String xml = "<resource-delta />";
		XStream xstream = this.createXStreamInstance( );
		Object deserialised = xstream.fromXML( xml );

		assertNotNull( deserialised );
		assertEquals( ResourceDeltaInformation.class , deserialised.getClass( ) );
		this.rdi = (ResourceDeltaInformation) deserialised;

		assertNull( this.rdi.getResource( ) );
		assertNull( this.rdi.getIncome( ) );
		assertNull( this.rdi.getUpkeep( ) );
	}


	/**
	 * Create and set up the {@link XStream} instance used in the serialisation tests
	 * 
	 * @return the {@link XStream} instance to use
	 */
	private XStream createXStreamInstance( )
	{
		XStream xstream = new XStream( );
		xstream.processAnnotations( ResourceDeltaInformation.class );
		return xstream;
	}
}
