package com.deepclone.lw.beans.bt.es.data;


import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Before;
import org.junit.Test;

import com.thoughtworks.xstream.XStream;



/**
 * Tests for the {@link ResourceProviderInformation} XML dump storage class
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestResourceProviderInformation
{
	/** A string used in tests */
	private static final String TEST_STRING = "This is a test";

	/** A real number used in tests */
	private static final Double TEST_DOUBLE = 4.2;

	/** Mining priority value used in tests */
	private static final Integer TEST_PRIORITY = 3;

	/** A resource provider information record */
	private ResourceProviderInformation rpi;


	@Before
	public void setUp( )
	{
		this.rpi = new ResourceProviderInformation( );
	}


	/**
	 * A new record's fields are all set to <code>null</code>.
	 */
	@Test
	public void testDefaultValues( )
	{
		assertNull( this.rpi.getResource( ) );
		assertNull( this.rpi.getCurrentQuantity( ) );
		assertNull( this.rpi.getMaximalQuantity( ) );
		assertNull( this.rpi.getDifficulty( ) );
		assertNull( this.rpi.getRecovery( ) );
		assertNull( this.rpi.getMiningPriority( ) );
	}


	/**
	 * Setting and reading the resource's name
	 */
	@Test
	public void testResourceName( )
	{
		this.rpi.setResource( TEST_STRING );
		assertEquals( TEST_STRING , this.rpi.getResource( ) );
	}


	/**
	 * Setting the resource name to <code>null</code> throws {@link InvalidDumpContentsException}
	 */
	@Test
	public void testNullResourceName( )
	{
		try {
			this.rpi.setResource( null );
			fail( "No InvalidDumpContentsException thrown" );
		} catch ( InvalidDumpContentsException exception ) {
			assertEquals( ResourceProviderInformation.class , exception.getRecordType( ) );
			assertEquals( "resource" , exception.getField( ) );
		}
	}


	/**
	 * Setting and reading the current quantity
	 */
	@Test
	public void testCurrentQuantity( )
	{
		this.rpi.setCurrentQuantity( TEST_DOUBLE );
		assertEquals( TEST_DOUBLE , this.rpi.getCurrentQuantity( ) );
	}


	/**
	 * Setting and reading the maximal quantity
	 */
	@Test
	public void testMaximalQuantity( )
	{
		this.rpi.setMaximalQuantity( TEST_DOUBLE );
		assertEquals( TEST_DOUBLE , this.rpi.getMaximalQuantity( ) );
	}


	/**
	 * Setting and reading the extraction difficulty
	 */
	@Test
	public void testDifficulty( )
	{
		this.rpi.setDifficulty( TEST_DOUBLE );
		assertEquals( TEST_DOUBLE , this.rpi.getDifficulty( ) );
	}


	/**
	 * Setting and reading the recovery rate
	 */
	@Test
	public void testRecovery( )
	{
		this.rpi.setRecovery( TEST_DOUBLE );
		assertEquals( TEST_DOUBLE , this.rpi.getRecovery( ) );
	}


	/**
	 * Setting and reading the mining priority
	 */
	@Test
	public void testMiningPriority( )
	{
		this.rpi.setMiningPriority( TEST_PRIORITY );
		assertEquals( TEST_PRIORITY , this.rpi.getMiningPriority( ) );
	}


	/**
	 * Serialising the instance to XML, without mining priority
	 */
	@Test
	public void testXMLSerialisation( )
	{
		this.rpi.setResource( TEST_STRING );
		this.rpi.setCurrentQuantity( 0.1 );
		this.rpi.setMaximalQuantity( 0.2 );
		this.rpi.setDifficulty( 0.3 );
		this.rpi.setRecovery( 0.4 );

		XStream xstream = this.createXStreamInstance( );
		String serialised = xstream.toXML( this.rpi );
		assertNotNull( serialised );
		assertTrue( serialised.startsWith( "<resource-provider " ) );
		assertTrue( serialised.endsWith( "/>" ) );
		assertTrue( serialised.contains( " resource=\"" + TEST_STRING + "\"" ) );
		assertTrue( serialised.contains( " current=\"0.1\"" ) );
		assertTrue( serialised.contains( " max=\"0.2\"" ) );
		assertTrue( serialised.contains( " difficulty=\"0.3\"" ) );
		assertTrue( serialised.contains( " recovery=\"0.4\"" ) );
		assertFalse( serialised.contains( "mining-priority=" ) );
	}


	/**
	 * Serialising the instance to XML, with a mining priority
	 */
	@Test
	public void testXMLSerialisationWithPriority( )
	{
		this.rpi.setResource( TEST_STRING );
		this.rpi.setCurrentQuantity( 0.1 );
		this.rpi.setMaximalQuantity( 0.2 );
		this.rpi.setDifficulty( 0.3 );
		this.rpi.setRecovery( 0.4 );
		this.rpi.setMiningPriority( TEST_PRIORITY );

		XStream xstream = this.createXStreamInstance( );
		String serialised = xstream.toXML( this.rpi );
		assertNotNull( serialised );
		assertTrue( serialised.startsWith( "<resource-provider " ) );
		assertTrue( serialised.endsWith( "/>" ) );
		assertTrue( serialised.contains( " resource=\"" + TEST_STRING + "\"" ) );
		assertTrue( serialised.contains( " current=\"0.1\"" ) );
		assertTrue( serialised.contains( " max=\"0.2\"" ) );
		assertTrue( serialised.contains( " difficulty=\"0.3\"" ) );
		assertTrue( serialised.contains( " recovery=\"0.4\"" ) );
		assertTrue( serialised.contains( " mining-priority=\"" + TEST_PRIORITY.toString( ) + "\"" ) );
	}


	/**
	 * Deserialising an instance - no mining priority
	 */
	@Test
	public void testXMLDeserialisation( )
	{
		String xml = "<resource-provider resource=\"Test\" current=\"0.1\" max=\"0.2\" difficulty=\"0.3\" recovery=\"0.4\" />";
		XStream xstream = this.createXStreamInstance( );
		Object deserialised = xstream.fromXML( xml );

		assertNotNull( deserialised );
		assertEquals( ResourceProviderInformation.class , deserialised.getClass( ) );
		this.rpi = (ResourceProviderInformation) deserialised;

		assertEquals( "Test" , this.rpi.getResource( ) );
		assertEquals( (Double) 0.1 , this.rpi.getCurrentQuantity( ) );
		assertEquals( (Double) 0.2 , this.rpi.getMaximalQuantity( ) );
		assertEquals( (Double) 0.3 , this.rpi.getDifficulty( ) );
		assertEquals( (Double) 0.4 , this.rpi.getRecovery( ) );
		assertNull( this.rpi.getMiningPriority( ) );
	}


	/**
	 * Deserialising an instance with mining priority
	 */
	@Test
	public void testXMLDeserialisationWithPriority( )
	{
		String xml = "<resource-provider resource=\"" + TEST_STRING
				+ "\" current=\"0.1\" max=\"0.2\" difficulty=\"0.3\" recovery=\"0.4\" mining-priority=\""
				+ TEST_PRIORITY.toString( ) + "\" />";
		XStream xstream = this.createXStreamInstance( );
		Object deserialised = xstream.fromXML( xml );

		assertNotNull( deserialised );
		assertEquals( ResourceProviderInformation.class , deserialised.getClass( ) );
		this.rpi = (ResourceProviderInformation) deserialised;

		assertEquals( TEST_STRING , this.rpi.getResource( ) );
		assertEquals( (Double) 0.1 , this.rpi.getCurrentQuantity( ) );
		assertEquals( (Double) 0.2 , this.rpi.getMaximalQuantity( ) );
		assertEquals( (Double) 0.3 , this.rpi.getDifficulty( ) );
		assertEquals( (Double) 0.4 , this.rpi.getRecovery( ) );
		assertEquals( TEST_PRIORITY , this.rpi.getMiningPriority( ) );

	}


	/**
	 * Create and set up the {@link XStream} instance used in the serialisation tests
	 * 
	 * @return the {@link XStream} instance to use
	 */
	private XStream createXStreamInstance( )
	{
		XStream xstream = new XStream( );
		xstream.processAnnotations( ResourceProviderInformation.class );
		xstream.alias( "resource-provider" , ResourceProviderInformation.class );
		return xstream;
	}
}
