package com.deepclone.lw.beans.updates;


import static org.junit.Assert.*;

import java.sql.Timestamp;
import java.util.Date;

import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.interfaces.game.updates.UpdateBatchProcessor;
import com.deepclone.lw.interfaces.sys.MaintenanceData;
import com.deepclone.lw.testing.MockLogger;
import com.deepclone.lw.testing.MockSystemStatus;
import com.deepclone.lw.testing.MockTransactionManager;



/**
 * Tests for {@link GameUpdateProcessorBean}
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestGameUpdateProcessorBean
{

	/** The mock logger */
	private MockLogger logger;

	/** Mock updates access interface */
	private MockUpdatesDAO updatesDAO;

	/** Mock processor registry */
	private MockRegistry registry;

	/** Mock system status component */
	private MockSystemStatus system;

	/** The instance under test */
	private GameUpdateProcessorBean gup;

	/**
	 * A fake batch processor which actually enables maintenance mode.
	 * 
	 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
	 */
	private static class MaintenanceEnabler
			implements UpdateBatchProcessor
	{

		/** Mock system status component */
		private MockSystemStatus system;


		/**
		 * Set the mock system status component
		 * 
		 * @param system
		 *            the mock system status component
		 */
		MaintenanceEnabler( MockSystemStatus system )
		{
			this.system = system;
		}


		@Override
		public String getUpdateType( )
		{
			return "maintenance";
		}


		@Override
		public void processBatch( long tickId )
		{
			this.system.setMaintenance( new MaintenanceData( new Timestamp( new Date( ).getTime( ) ) , new Timestamp(
					new Date( ).getTime( ) ) , "no reason" ) );
		}

	}


	@Before
	public void setUp( )
	{
		this.logger = new MockLogger( );
		this.updatesDAO = new MockUpdatesDAO( );
		this.updatesDAO.values = new String[] {
			null
		};
		this.system = new MockSystemStatus( );
		this.registry = new MockRegistry( );
		this.registry.put( new MaintenanceEnabler( this.system ) );

		this.gup = new GameUpdateProcessorBean( );
		this.gup.setLogger( this.logger );
		this.gup.setUpdatesDAO( this.updatesDAO );
		this.gup.setRegistry( this.registry );
		this.gup.setTransactionManager( new MockTransactionManager( ) );
		this.gup.setSystemStatus( this.system );
	}


	/**
	 * Try locking the processor
	 */
	@Test
	public void testLocking( )
	{
		assertTrue( this.gup.tryLock( ) );
		assertFalse( this.gup.tryLock( ) );
	}


	/** Try unlocking the processor */
	@Test
	public void testUnlocking( )
	{
		this.gup.tryLock( );
		this.gup.unlock( );
		assertTrue( this.gup.tryLock( ) );
	}


	/** Try unlocking the processor when it's not locked */
	@Test( expected = IllegalStateException.class )
	public void testUnlockingWithNoLock( )
	{
		this.gup.unlock( );
	}


	/**
	 * {@link GameUpdateProcessorBean#endPreviousCycle()} returns <code>false</code> when there is
	 * no "stuck" tick
	 */
	@Test
	public void testEndCycleNoStuckTick( )
	{
		assertFalse( this.gup.endPreviousCycle( ) );
	}


	/**
	 * {@link GameUpdateProcessorBean#endPreviousCycle()} returns <code>true</code> when maintenance
	 * mode is enabled, but the update is not processed
	 */
	@Test
	public void testEndCycleMaintenance( )
	{
		this.system.setCSTBehaviour( -2 );
		assertTrue( this.gup.endPreviousCycle( ) );
		assertEquals( 0 , this.updatesDAO.index );
	}


	/**
	 * {@link GameUpdateProcessorBean#endPreviousCycle()} returns <code>true</code> when there is a
	 * stuck tick, and the update is processed
	 */
	@Test
	public void testEndCycle( )
	{
		this.system.setCSTBehaviour( 0 );
		assertTrue( this.gup.endPreviousCycle( ) );
		assertEquals( 1 , this.updatesDAO.index );
	}


	/**
	 * {@link GameUpdateProcessorBean#executeUpdateCycle()} does not start a tick if there was a
	 * stuck tick, but the previous tick is processed.
	 */
	@Test
	public void testProcessWithStuckTick( )
	{
		this.system.setCSTBehaviour( 0 );
		this.gup.executeUpdateCycle( );
		assertFalse( this.system.wasStartTickCalled( ) );
		assertEquals( 1 , this.updatesDAO.index );
	}


	/**
	 * {@link GameUpdateProcessorBean#executeUpdateCycle()} does not start a tick if maintenance
	 * mode was enabled from the start
	 */
	@Test
	public void testProcessWithStuckTickAndMaintenance( )
	{
		this.system.setCSTBehaviour( -2 );
		this.gup.executeUpdateCycle( );
		assertFalse( this.system.wasStartTickCalled( ) );
		assertEquals( 0 , this.updatesDAO.index );
	}


	/**
	 * {@link GameUpdateProcessorBean#executeUpdateCycle()} does not start a tick if maintenance
	 * mode is enabled between the check for stuck ticks and the new tick's start.
	 */
	@Test
	public void testProcessWithMaintenance( )
	{
		this.system.setSTBehaviour( -2 );
		this.gup.executeUpdateCycle( );
		assertTrue( this.system.wasStartTickCalled( ) );
		assertEquals( 0 , this.updatesDAO.index );
	}


	/**
	 * {@link GameUpdateProcessorBean#executeUpdateCycle()} throws a runtime exception if there
	 * seems to be a tick running when the new tick is supposed to start.
	 */
	@Test( expected = RuntimeException.class )
	public void testProcessWithBadState( )
	{
		this.system.setSTBehaviour( -1 );
		this.gup.executeUpdateCycle( );
	}


	/**
	 * {@link GameUpdateProcessorBean#executeUpdateCycle()} stops processing mid-way if maintenance
	 * mode becomes enabled
	 */
	@Test
	public void testProcessWithMaintenanceEnabledMidWay( )
	{
		this.updatesDAO.values = new String[] {
				null , "maintenance" , null
		};
		this.gup.executeUpdateCycle( );
		assertTrue( this.system.wasStartTickCalled( ) );
		assertEquals( 2 , this.updatesDAO.index );
	}
}
