package com.deepclone.lw.cli.xmlimport;


import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;

import org.junit.Test;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.resources.BasicResource;
import com.deepclone.lw.cli.xmlimport.data.resources.Resources;
import com.thoughtworks.xstream.converters.ConversionException;
import com.thoughtworks.xstream.io.StreamException;



/**
 * Unit tests for the {@link ResourceLoader} class.
 * 
 * @author <a href="mailto:tim@mitheren.com">T. Rosser</a>
 * 
 */
public class TestResourceLoader
{
	/**
	 * Try initialising the loader with a <code>null</code> file instance.
	 * 
	 * @throws NullPointerException
	 *             when the constructor is executed
	 */
	@Test( expected = NullPointerException.class )
	public void testNullFile( )
			throws NullPointerException
	{
		new ResourceLoader( null );
	}


	/** Try loading a file that does not exist */
	@Test
	public void testMissingFile( )
	{
		ResourceLoader loader = new ResourceLoader( new File( "does-not-exist" ) );
		try {
			loader.load( );
		} catch ( DataImportException e ) {
			assertTrue( "cause is a" + e.getCause( ).getClass( ).getName( ) , e.getCause( ) instanceof IOException );
			return;
		}
		fail( "no exception after trying to load a missing file" );
	}


	/** Try loading a file that contains something that is not XML. */
	@Test
	public void testBadXML( )
	{
		ResourceLoader loader = new ResourceLoader( new File( "TestFiles/resource-loader/bad-xml.xml" ) );
		try {
			loader.load( );
		} catch ( DataImportException e ) {
			assertTrue( "cause is a " + e.getCause( ).getClass( ).getName( ) , e.getCause( ) instanceof StreamException );
			return;
		}
		fail( "no exception after loading stuff that isn't XML" );
	}


	/**
	 * Test loading a file that contains XML but which cannot be deserialised to an
	 * {@link Resources} instance.
	 */
	@Test
	public void testBadContents( )
	{
		ResourceLoader loader = new ResourceLoader( new File( "TestFiles/resource-loader/bad-contents.xml" ) );
		try {
			loader.load( );
		} catch ( DataImportException e ) {
			assertTrue( "cause is a " + e.getCause( ).getClass( ).getName( ) ,
					e.getCause( ) instanceof ConversionException );
			return;
		}
		fail( "no exception after loading bad XML" );
	}


	/**
	 * Try loading a file that contains valid XML for a {@link ResourceLoader} instance with
	 * semantic errors.
	 */
	@Test
	public void testBadData( )
	{
		ResourceLoader loader = new ResourceLoader( new File( "TestFiles/resource-loader/bad-data.xml" ) );
		try {
			loader.load( );
		} catch ( DataImportException e ) {
			assertNull( e.getCause( ) );
			return;
		}
		fail( "no exception after loading bad data" );
	}


	/** Try loading valid data, make sure that it contains exactly what is expected */
	@Test
	public void testGoodData( )
	{
		ResourceLoader loader = new ResourceLoader( new File( "TestFiles/resource-loader/good-data.xml" ) );
		Resources resources;
		try {
			resources = loader.load( );
		} catch ( DataImportException e ) {
			fail( "could not load valid file" );
			return;
		}
		assertNotNull( resources );

		// Not sure if this is the best way to code for the two different resource types...
		int rCount = 0;

		for ( @SuppressWarnings( "unused" )
		BasicResource br : resources ) {
			rCount++;
		}
		assertEquals( 2 , rCount );
	}
}
