package com.deepclone.lw.cli.xmlimport.data.i18n;


import java.io.File;
import java.lang.reflect.Method;

import org.junit.Test;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.ImportableData;

import static org.junit.Assert.*;



/**
 * Tests for the {@link InlineString} class
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestInlineString
		extends BaseTest
{

	/** Test loading a valid in-line string record */
	@Test
	public void testLoadStringOK( )
	{
		String definition = this.createInlineStringDefinition( "test" , "Test" );
		InlineString instance = this.createObject( definition , InlineString.class );
		assertEquals( "test" , instance.getId( ) );
		assertEquals( "Test" , instance.getString( ) );
	}


	/** Test loading an in-line string record without identifier */
	@Test
	public void testLoadStringNoIdentifier( )
	{
		String definition = this.createInlineStringDefinition( null , "Test" );
		InlineString instance = this.createObject( definition , InlineString.class );
		assertNull( instance.getId( ) );
		assertEquals( "Test" , instance.getString( ) );
	}


	/** Test loading an in-line string record without contents */
	@Test
	public void testLoadStringNoContents( )
	{
		String definition = this.createInlineStringDefinition( "test" , null );
		InlineString instance = this.createObject( definition , InlineString.class );
		assertEquals( "test" , instance.getId( ) );
		assertNull( instance.getString( ) );
	}


	/** Test validating an in-line string record with a short identifier and some contents */
	@Test
	public void testValidateSimple( )
			throws DataImportException
	{
		String definition = this.createInlineStringDefinition( "test" , "Test" );
		InlineString instance = this.createObject( definition , InlineString.class );
		instance.verifyData( );
	}


	/** Test validating an in-line string record with a camel case identifier and some contents */
	@Test
	public void testValidateCamelCaseIdentifier( )
			throws DataImportException
	{
		String definition = this.createInlineStringDefinition( "testCamelCase" , "Test" );
		InlineString instance = this.createObject( definition , InlineString.class );
		instance.verifyData( );
	}


	/**
	 * Test validating an in-line string record with an identifier that includes a sequence of
	 * upper-case letters and some contents
	 */
	@Test
	public void testValidateCapsIdentifier( )
			throws DataImportException
	{
		String definition = this.createInlineStringDefinition( "testCAPSIdentifier" , "Test" );
		InlineString instance = this.createObject( definition , InlineString.class );
		instance.verifyData( );
	}


	/**
	 * Test validating an in-line string record with an identifier that includes numbers and some
	 * contents
	 */
	@Test
	public void testValidateNumbersIdentifier( )
			throws DataImportException
	{
		String definition = this.createInlineStringDefinition( "test123Numbers123" , "Test" );
		InlineString instance = this.createObject( definition , InlineString.class );
		instance.verifyData( );
	}


	/**
	 * Test rejecting an in-line string record with a <code>null</code> identifier and some contents
	 */
	@Test( expected = DataImportException.class )
	public void testValidateNullIdentifier( )
			throws DataImportException
	{
		String definition = this.createInlineStringDefinition( null , "Test" );
		InlineString instance = this.createObject( definition , InlineString.class );
		instance.verifyData( );
	}


	/** Test rejecting an in-line string record with an identifier that starts with a bad character */
	@Test( expected = DataImportException.class )
	public void testValidateBadFirstCharacterIdentifier( )
			throws DataImportException
	{
		String definition = this.createInlineStringDefinition( " test" , "Test" );
		InlineString instance = this.createObject( definition , InlineString.class );
		instance.verifyData( );
	}


	/**
	 * Test rejecting an in-line string record with an identifier that starts with an upper-case
	 * letter
	 */
	@Test( expected = DataImportException.class )
	public void testValidateFirstCharacterWrongCaseIdentifier( )
			throws DataImportException
	{
		String definition = this.createInlineStringDefinition( "Test" , "Test" );
		InlineString instance = this.createObject( definition , InlineString.class );
		instance.verifyData( );
	}


	/** Test rejecting an in-line string record with an identifier that includes "bad" characters */
	@Test( expected = DataImportException.class )
	public void testValidateBadMiddleCharacterIdentifier( )
			throws DataImportException
	{
		String definition = this.createInlineStringDefinition( "test Test" , "Test" );
		InlineString instance = this.createObject( definition , InlineString.class );
		instance.verifyData( );
	}


	/** Test rejecting an in-line record with <code>null</code> contents */
	@Test( expected = DataImportException.class )
	public void testValidateNullContents( )
			throws DataImportException
	{
		String definition = this.createInlineStringDefinition( "test" , null );
		InlineString instance = this.createObject( definition , InlineString.class );
		instance.verifyData( );
	}


	/** Test rejecting an in-line record with empty contents */
	@Test( expected = DataImportException.class )
	public void testValidateEmptyContents( )
			throws DataImportException
	{
		String definition = this.createInlineStringDefinition( "test" , "" );
		InlineString instance = this.createObject( definition , InlineString.class );
		instance.verifyData( );
	}


	/**
	 * Test rejecting an in-line record with contents the include only white space (spaces, tabs,
	 * new line, line feed)
	 */
	@Test( expected = DataImportException.class )
	public void testValidateSpacesOnlyContents( )
			throws DataImportException
	{
		String definition = this.createInlineStringDefinition( "test" , " \t\r\n" );
		InlineString instance = this.createObject( definition , InlineString.class );
		instance.verifyData( );
	}


	/**
	 * Make sure the class does not override {@link ImportableData#setReadFrom(java.io.File)}; if it
	 * does, new tests <strong>must</strong> be written.
	 * 
	 * @throws NoSuchMethodException
	 *             if the method has been deleted
	 * @throws SecurityException
	 *             if the JVM set-up is getting in the wayy
	 */
	@Test
	public void testNoSetReadFromOverride( )
			throws SecurityException , NoSuchMethodException
	{
		Class< InlineString > cls = InlineString.class;
		Method method = cls.getMethod( "setReadFrom" , File.class );
		assertEquals( ImportableData.class , method.getDeclaringClass( ) );
	}
}
