package com.deepclone.lw.cli.xmlimport.data.i18n;


import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;



/**
 * Unit tests for the {@link LanguageDefinition} class
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestLanguageDefinition
		extends BaseTest
{

	/** The in-line string definition that will be used in most tests */
	private String inlineDefinition;


	/**
	 * Create a valid in-line string definition to be appended to language definitions.
	 */
	@Before
	public void setUp( )
	{
		this.inlineDefinition = this.createInlineStringDefinition( "test" , "test" );
	}


	/**
	 * Test loading a valid language definition and make sure that all fields were set
	 * appropriately.
	 * 
	 * @throws DataImportException
	 *             if accessing the test string fails.
	 */
	@Test
	public void testLoadValidDefinition( )
			throws DataImportException
	{
		String xml = this.createLanguageDefinition( "t" , "Test" , this.inlineDefinition );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		assertEquals( "t" , language.getId( ) );
		assertEquals( "Test" , language.getName( ) );

		int count = 0;
		for ( StringDefinition stringDefinition : language ) {
			assertEquals( InlineString.class , stringDefinition.getClass( ) );
			assertEquals( "test" , stringDefinition.getId( ) );
			assertEquals( "test" , stringDefinition.getString( ) );
			count++;
		}
		assertEquals( 1 , count );
	}


	/**
	 * Test loading a language definition that has no identifier.
	 */
	@Test
	public void testLoadNullIdentifier( )
	{
		String xml = this.createLanguageDefinition( null , "Test" , this.inlineDefinition );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		assertNull( language.getId( ) );
	}


	/**
	 * Test loading a language definition that has no name
	 */
	@Test
	public void testLoadNullName( )
	{
		String xml = this.createLanguageDefinition( "test" , null , this.inlineDefinition );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		assertNull( language.getName( ) );
	}


	/**
	 * Test loading a language definition that has no contents
	 * 
	 * @throws NullPointerException
	 *             when the language definition's iterator is accessed, as its contents will be
	 *             <code>null</code>
	 */
	@Test( expected = NullPointerException.class )
	public void testLoadNullContents( )
			throws NullPointerException
	{
		String xml = this.createLanguageDefinition( "test" , "test" );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		language.iterator( );
	}


	/**
	 * Test validating a correct language definition.
	 * 
	 * @throws DataImportException
	 *             if validating the definition fails
	 */
	@Test
	public void testValidateOK( )
			throws DataImportException
	{
		String xml = this.createLanguageDefinition( "t" , "Test" , this.inlineDefinition );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		language.verifyData( );
	}


	/**
	 * Test validating a language definition with a <code>null</code> identifier.
	 * 
	 * @throws DataImportException
	 *             when validation fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateNullIdentifier( )
			throws DataImportException
	{
		String xml = this.createLanguageDefinition( null , "Test" , this.inlineDefinition );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		language.verifyData( );
	}


	/**
	 * Test validating a language definition with an empty identifier.
	 * 
	 * @throws DataImportException
	 *             when validation fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateEmptyIdentifier( )
			throws DataImportException
	{
		String xml = this.createLanguageDefinition( "" , "Test" , this.inlineDefinition );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		language.verifyData( );
	}


	/**
	 * Test validating a language definition with an identifier that contains white space only.
	 * 
	 * @throws DataImportException
	 *             when validation fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateWhiteSpaceIdentifier( )
			throws DataImportException
	{
		String xml = this.createLanguageDefinition( " \t\n\r" , "Test" , this.inlineDefinition );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		language.verifyData( );
	}


	/**
	 * Test validating a language definition with a <code>null</code> name.
	 * 
	 * @throws DataImportException
	 *             when validation fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateNullName( )
			throws DataImportException
	{
		String xml = this.createLanguageDefinition( "t" , null , this.inlineDefinition );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		language.verifyData( );
	}


	/**
	 * Test validating a language definition with an empty name.
	 * 
	 * @throws DataImportException
	 *             when validation fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateEmptyName( )
			throws DataImportException
	{
		String xml = this.createLanguageDefinition( "t" , "" , this.inlineDefinition );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		language.verifyData( );
	}


	/**
	 * Test validating a language definition with a name that contains white space only.
	 * 
	 * @throws DataImportException
	 *             when validation fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateWhiteSpaceName( )
			throws DataImportException
	{
		String xml = this.createLanguageDefinition( "t" , " \t\n\r" , this.inlineDefinition );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		language.verifyData( );
	}


	/**
	 * Test validating a language definition with no contents.
	 * 
	 * @throws DataImportException
	 *             when validation fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateNoContents( )
			throws DataImportException
	{
		String xml = this.createLanguageDefinition( "t" , "Test" );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		language.verifyData( );
	}


	/**
	 * Test validating a language definition with invalid contents.
	 * 
	 * @throws DataImportException
	 *             when validation fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateBadContents( )
			throws DataImportException
	{
		String strXML = this.createInlineStringDefinition( null , "test" );
		String xml = this.createLanguageDefinition( "t" , "Test" , strXML );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		language.verifyData( );
	}


	/**
	 * Test validating a language definition with duplicate contents.
	 * 
	 * @throws DataImportException
	 *             when validation fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateDuplicateContents( )
			throws DataImportException
	{
		String xml = this.createLanguageDefinition( "t" , "Test" , this.inlineDefinition , this.inlineDefinition );
		LanguageDefinition language = this.createObject( xml , LanguageDefinition.class );
		language.verifyData( );
	}

}
