package com.deepclone.lw.web.main.game;


import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.PlayerSession;



/**
 * Overview page controller
 * 
 * <p>
 * This page controller implements the "Overview" page, as well as the commands it supports
 * (implement technology, update empire mining settings).
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@Controller
@SessionRequirement( value = true , redirectTo = "player-session" , subType = "game" )
@SessionAttributes( "language" )
public class OverviewPage
		extends PageControllerBase
{

	/**
	 * Main overview display
	 * 
	 * <p>
	 * This method is mapped to the overview page's URL. It will fetch the empire overview from the
	 * server and display the appropriate page.
	 * 
	 * @param request
	 *            the HTTP request
	 * @param language
	 *            the language from the session
	 * @param model
	 *            the model
	 * 
	 * @return the overview page rendering order
	 * 
	 * @throws SessionException
	 *             if some error occurs on the server
	 * @throws SessionServerException
	 *             if the server is unreachable
	 * @throws SessionMaintenanceException
	 *             if the game is under maintenance
	 */
	@RequestMapping( "/overview" )
	public String overview( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "overview" , pSession.getOverview( ) );
	}


	/**
	 * Empire-wide mining settings update command
	 * 
	 * <p>
	 * This method is called when a command to update empire-wide mining settings is received.
	 * 
	 * @param request
	 *            the HTTP request
	 * 
	 * @return a redirection to the overview page's "economy" tab
	 * 
	 * @throws SessionException
	 *             if some error occurs on the server
	 * @throws SessionServerException
	 *             if the server is unreachable
	 * @throws SessionMaintenanceException
	 *             if the game is under maintenance
	 */
	@RequestMapping( value = "/update-mining-settings.action" , method = RequestMethod.POST )
	public String updateMiningSettings( HttpServletRequest request )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Map< String , Integer > miningSettings = this.getMiningSettings( request );
		if ( miningSettings != null ) {
			this.getSession( PlayerSession.class , request ).updateEmpireMiningSettings( miningSettings );
		}
		return this.redirect( "overview#economy" );
	}


	/**
	 * Extract mining priorities from the HTTP request
	 * 
	 * <p>
	 * Look for all submitted fields that begin with "ems-" then try to extract their values into a
	 * map that associates resource identifiers to priorities.
	 * 
	 * @param request
	 *            the HTTP request
	 * 
	 * @return the map containing the submitted mining settings, or <code>null</code> if one of the
	 *         values was incorrect.
	 */
	private Map< String , Integer > getMiningSettings( HttpServletRequest request )
	{
		Map< String , Object > input = this.getInput( request );
		Map< String , Integer > miningSettings = new HashMap< String , Integer >( );
		for ( Entry< String , Object > entry : input.entrySet( ) ) {
			// Ignore items which are not mining settings
			String name = entry.getKey( );
			if ( !name.startsWith( "ems-" ) ) {
				continue;
			}
			name = name.substring( 4 );

			// Get values
			if ( ! ( entry.getValue( ) instanceof String[] ) ) {
				continue;
			}
			String[] values = (String[]) entry.getValue( );
			if ( values.length < 1 ) {
				continue;
			}

			// Pre-validate them
			int value;
			try {
				value = Integer.parseInt( values[ 0 ] );
			} catch ( NumberFormatException e ) {
				value = -1;
			}
			if ( value < 0 || value > 4 ) {
				return null;
			}

			miningSettings.put( name , value );
		}
		return miningSettings;
	}
}
